@   Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
@   Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
@
@   This file is part of Jespa3D Engine.
@
@   Jespa3D Engine is free software; you can redistribute it and/or modify
@   it under the terms of the GNU General Public License as published by
@   the Free Software Foundation; either version 2 of the License, or
@   (at your option) any later version.
@
@   Jespa3D Engine is distributed in the hope that it will be useful,
@   but WITHOUT ANY WARRANTY; without even the implied warranty of
@   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@   GNU General Public License for more details.
@
@   You should have received a copy of the GNU General Public License
@   along with Jespa3D Engine; ; see the file COPYING. if not, write to 
@   the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
@   Boston, MA  02111-1307  USA


@ /***********************\
@ | PROTOTIPUS DE FUNCIONS |
@ \***********************/





@ /***********************\
@ | DEFINICIO DE CONSTANTS |
@ \***********************/

.CODE 32
.SECTION .iwram
.ALIGN


.GLOBL  ARM_OneOverFx
.GLOBL  ARM_MulFx32
.GLOBL  ARM_MulFx24
.GLOBL  ARM_DivFx_Entera
.GLOBL  ARM_DivFx24
.GLOBL  ARM_DivFx32
.GLOBL  ARM_iDiv64
.GLOBL  ARM_SqrtFxLP
.GLOBL  ARM_SquareFx

.TYPE   ARM_OneOverFx, function
.TYPE   ARM_MulFx32, function
.TYPE   ARM_MulFx24, function
.TYPE   ARM_DivFx_Entera, function
.TYPE   ARM_DivFx24, function
.TYPE   ARM_DivFx32, function
.TYPE   ARM_iDiv64, function
.TYPE   ARM_SqrtFxLP, function
.TYPE   ARM_SquareFx, function


@ /***************************\
@ | IMPLEMENTACIO DE FUNCIONS |
@ \***************************/


@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@
@ LA MULTIPLICACI DE COME FIXE...
@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


ARM_MulFx32:

        @
        @ Entrades
        @ --------
        @
        @ r0 = Operant 1.
        @ r1 = Operant 2.
        @
        @ Sortida
        @ -------
        @
        @ r0 = Resultat de la multiplicaci.
        @

        STMFD   sp!, {r3-r5}         @ Save registers				

		SMULL   r4,r5,r1,r0		@ (r5:r4) <- r2 * r1 (Part baixa del resultat es guardar a r4 i a r5 la part alta.
		ADDS    r4,r4,#0x8000		@ Sumem r4 <- r4 + 0x8000 habilitant l`opci de modificar el registre CPSR
		ADC     r5,r5,#0			@ r5 <- r5 + BitDeCarryDeR4

        MOV     r4,r4,LSR #16       @ La part baixa de R4 cont la part alta de R4.
        MOV     r5,r5,LSL #16       @ La part alta de R5 cont la part baixa de R5.

        ORR     r4,r4,r5            @ Or logica sense comptar amb el bit de carry.
			
        MOV     r0,r4               @ r0 <- r4 (Registre de retorn)
        
        LDMFD   sp!, {r3-r5}        @ Restore registers				
        BX      lr


ARM_SquareFx:

        @
        @ Entrades
        @ --------
        @
        @ r0 = Operant 1.
        @
        @ Sortida
        @ -------
        @
        @ r0 = Resultat del quadrat.
        @

        STMFD   sp!, {r4-r5}         @ Save registers		

		SMULL   r4,r5,r0,r0			@ (r5:r4) <- r1 * r1 (Part baixa del resultat es guardar a r4 i a r5 la part alta.
		ADDS    r4,r4,#0x8000		@ Sumem r4 <- r4 + 0x8000 habilitant l`opci de modificar el registre CPSR
		ADC     r5,r5,#0			@ r5 <- r5 + BitDeCarryDeR4
        MOV     r4,r4,LSR #16       @ La part baixa de R4 cont la part alta de R4.
        MOV     r5,r5,LSL #16       @ La part alta de R5 cont la part baixa de R5.

        ORR     r4,r4,r5            @ Or logica sense comptar amb el bit de carry.
			
        MOV     r0,r4               @ r0 <- r4
        
        LDMFD   sp!, {r4-r5}        @ Restore registers				
        BX      lr


ARM_MulFx24:

@
@ Entrades
@ --------
@
@ r0 = Operant 1.
@ r1 = Operant 2.
@
@ Sortida
@ -------
@
@ r0 = Resultat de la multiplicaci.
@



	MUL    r2,r1,r0	     	     @ r2 <- r1 * r0
	MOV    r0,r2,ASR #8	     @ r0 <- r2 >> 8

    BX     lr




@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@
@ LA DIVISIO DE PUNT FIXE...
@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

ARM_DivFx32:


@
@  R0:R1 = Numerador, Registre de retorn.
@  R3 = Denominador
@
@	.word	__Divisio64
@ Guardem parametres entrada...


                  STMFD   sp!, {r1-r8,lr}        @ Save registers
                  
                   MOV     r3,r1                @ r3 Denominador
                   MOV     r2,r0               @ r2 Numerador

                   MOV     r0,r2,ASR #16       @ ASR (Aritmtic Shift Right). Realitza desplaaments mantenint el bit de signe.
                   MOV     r1,r2,LSL #16       @ La part alta de eax cont la part baixa de edx.


@ Divisi r0:r1 dividit per r3

                   BL	   ARM_iDiv64        @ Cridem la funci divisi de 64 bits.
                                                @ BL modifica r14, o sigui, el destrueix.


                   LDMFD   sp!, {r1-r8,lr}        @ Restore registers
                   
                   BX      lr

	
@					" :
@					"=r"(variable) :                        // El registre que entrega el valor a "variable" es R0.
@					"r" (Numerador), "r" (Denominador):     // Definim els parametres d' entrada (num1,num2) queden assignats als registre R1 i R2 respectivament.
@					"r0","r1","r2","r3","r4","r6","r7","r14");    // Especifiquem els registres que sern destruits.


ARM_DivFx24:

@ Entrades
@ --------
@
@  R0 = Numerador, Registre de retorn.
@  R1 = Denominador
@
@	
@ Guardem parametres entrada...

            MOV	   r0,r0,ASL #8

		   @ Executem la divisi de 32:32 bits mitjanant una interrupci SWI...

		   SWI	   0x60000                @ En mode ARM, el numero d'interrupci s
						  @ NN*10000h (NN s el numero hexadecimal de la interrupci)

           BX      lr


ARM_DivFx_Entera:

@ Entrades
@ --------
@
@  R0 = Numerador, Registre de retorn.
@  R1 = Denominador
@

            
	   @ Executem la divisi de 32:32 bits mitjanant una interrupci SWI...

	   SWI	   0x60000                @ En mode ARM, el numero d'interrupci s
					  @ NN*10000h (NN s el numero hexadecimal de la interrupci)

           BX      lr


	




@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@
@  ONE OVER DE PUNT FIXE...
@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

ARM_OneOverFx:


@
@  R0:R1 = Numerador, Registre de retorn.
@  R3 = Denominador
@
@	.word	__Divisio64
@ Guardem parametres entrada...


                  STMFD   sp!, {r1-r8,lr}        @ Save registers

                  MOV     r3,r0               @ Divisi r0:r1 div r3
                  MOV     r0,#1
                  EOR     r1,r1,r1		

@ r0:r1 dividit per r3


                   BL	   ARM_iDiv64           @ Cridem la funci divisi de 64 bits.
                                                @ BL modifica r14 -> El destrueix.


                   LDMFD   sp!, {r1-r8,lr}        @ Restore registers

                   BX      lr


@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@
@ L' ARREL QUADRADA DE BAIXA PRESICI (Low Precision)...
@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

ARM_SqrtFxLP:	

@
@  R0 = eax
@  R1 = ebx
@  R2 = ecx
@  R3 = edx
@
        STMFD   sp!, {r1-r2,lr}
		mov r2,r0           @ ecx <- num

		eor r0,r0,r0        @ eax <- 0
		mov	r1,#1073741824  @ ebx <-  40000000h

sqrtLP1:                    @ repetir

		 mov r3, r2
		 subs r3,r3, r1
		 bmi  sqrtLP2      @ salta si edx < ebx
		
         subs r3,r3, r0    @
		 bmi  sqrtLP2      @ salta si edx < eax
		
         mov r2, r3        @
		 mov r0,r0,LSR #1  @ eax <- eax div 2
		 orr  r0,r0, r1    @
		 movs r1, r1,LSR #2 @ ebx <- ebx div 4
		 bne sqrtLP1	   @ repetir mentre(ebx != 0)

		 mov r0,r0,LSL #8  @ eax <- eax * 256
		 b   sqrtLP3	   @

sqrtLP2:
		 mov r0,r0, LSR #1  @ eax <- eax div 2
		 movs r1,r1,LSR #2   @ ebx <- ebx div 4
		 bne sqrtLP1        @ repetir mentre(ebx != 0)

		 mov r0,r0,LSL #8   @ eax <- eax * 256

sqrtLP3: nop
		

         STMFD   sp!, {r1-r2,lr}
         BX      lr


