//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA

#ifndef __GRAFICS_H
 #define __GRAFICS_H

#include "definicions.h"
#include "stdlib.h"
#include "stdio.h"

/*************\
| DEFINICIONS |
\*************/


#ifdef __SDL__

    extern SDL_Surface *Video;

	#define SDL_WORK_VRAM  (char *)Video->pixels

    #define WORK_VRAM   SDL_WORK_VRAM

#endif

#ifdef __HAM__
    extern u16* VideoBuffer; // Es donde estamos dibujando actualmente 

    #define HAM_WORK_VRAM  VideoBuffer//((ham_GetBGBuffer() == 1) ? ((char*)0x06000000) : ((char*)0x0600A000))

    #define WORK_VRAM      HAM_WORK_VRAM
#endif




/***********************\
| DEFINICIONS DE TIPUS  |
\***********************/

typedef struct _tParametresPutSprite{
    
    int x_pos,y_pos,amplada_sprite,altura_sprite,mascara;
    char *src;

}tParametresPutSprite;




/*********\
| MACROS  |
\*********/




#ifdef __SDL__
 //
 // Per decodificar un color de la paleta GBA als valors R,G o B.
 // 

    #define RGB_GET_R_VALUE(rgb)    ((rgb & 0x001f) << 3)
    #define RGB_GET_G_VALUE(rgb)    (((rgb >> 5) & 0x001f) << 3)
    #define RGB_GET_B_VALUE(rgb)    (((rgb >> 10) & 0x001f) << 3)

#endif
	
	
void PutPixel(int x, int y, char color);	


/****************\
|MACROS GRAFIQUES |
\****************/


#ifdef __SDL__

//extern unsigned int _amblksiz;

	#define BORRAR_PANTALLA\
		if(SDL_MUSTLOCK(Video))\
		{\
				SDL_LockSurface(Video);\
		}\
		SDL_FillRect(Video,NULL,45/*rand()%256*/);\


	#define ESCRIURE_INFORMACIO_A_PANTALLA\
		SDL_UnlockSurface(Video);\
		SDL_Flip(Video);\

#endif


#ifdef __HAM__

///// define REG_DISPCNT

#define MODE_0 0x0
#define MODE_1 0x1
#define MODE_2 0x2
#define MODE_3 0x3
#define MODE_4 0x4
#define MODE_5 0x5

#define BACKBUFFER 0x10
#define H_BLANK_OAM 0x20

#define OBJ_MAP_2D 0x0
#define OBJ_MAP_1D 0x40

#define FORCE_BLANK 0x80

#define BG0_ENABLE 0x100
#define BG1_ENABLE 0x200 
#define BG2_ENABLE 0x400
#define BG3_ENABLE 0x800
#define OBJ_ENABLE 0x1000 
#define WIN1_ENABLE 0x2000 
#define WIN2_ENABLE 0x4000
#define WINOBJ_ENABLE 0x8000

// Esta definicin no es realmente necesaria ya que est definida en gba.h (de hecho probablemente dar un error)
//#define REG_DISPCNT * (u16*)0x4000000 //Define la direccin como un puntero y la referencia

///////Macro SetMode

#define SetMode(mode) REG_DISPCNT = (mode) 


#define FrontBuffer  (u16*)0x6000000
#define BackBuffer   (u16*)0x600A000 //Comienzo del buffer secundario en el modo 4 
#define paletteMem   (u16*)0x5000000

#define VCOUNT_SCALINE (volatile u16 *)0x4000006

#define Flip\
    {\
        while((*(VCOUNT_SCALINE)) < 160);\
        if(REG_DISPCNT & BACKBUFFER)\
        {\
                REG_DISPCNT &= ~BACKBUFFER;\
                VideoBuffer = BackBuffer;\
        }\
        else\
        {\
                REG_DISPCNT |= BACKBUFFER;\
                VideoBuffer = FrontBuffer;\
        }\
    }\

//GBA_LoadBGPal((u16 *)Palette,SIZEOF_16BIT(Palette));
#define GBA_LoadBGPal(Palette,n_hwords)  CopyMem_16bit(paletteMem, Palette, n_hwords)



#define BORRAR_PANTALLA				    MemSet16(VideoBuffer,0x0202,((240*160)>>1));    // NO es necessari borrar la pantalla pk a cada frame es pinten de nou tots els pixels (es guanya +1fps)
#define ESCRIURE_INFORMACIO_A_PANTALLA	Flip
#endif
	


/************************\
| PROTOTIPUS DE FUNCIONS |
\************************/
extern char *extra_info;

void Inicialitzar_SDL();
void Escriure_Info_Sortida();

#ifdef __HAM__
	extern void ARM_PutSprite(const tParametresPutSprite *P_PutSprite);
	#define PutSprite ARM_PutSprite
#endif

#ifdef __SDL__
	void SDL_PutSprite(const tParametresPutSprite  *P_PutSprite);
	#define PutSprite SDL_PutSprite
#endif

 




#endif



