//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA



////////////////////////////////////////
//									  //
// Inclusi de capaleres necessaries //
// __________________________________ //
//									  //
////////////////////////////////////////



#include "Linia.h"
#include "render.h"
#include "geometria.h"


///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
//
// Constructor per pas de parametres.
//


#ifndef __UTILITZAR_EL_MAPA_PREGUARDAT
void LINIA_InicialitzarLinia(int ID_Linia, tVertex V0, tVertex V1, u32 Propietats, tLinia *Linia)
{
	FIXED32			IncrementX, IncrementZ; 
	double alpha;
	int Quadrant;

	Linia->ID_Linia = ID_Linia;
	
	Linia->V0 = V0;
	Linia->V1 = V1;

	
	//{ S'inicializa la posici el quadrat de collisi.
	
	if(V0.X < V1.X) 
	{
		Linia->QuadratLinia.x1 = V0.X;
		Linia->QuadratLinia.x2 = V1.X;
	}
	else 
	{
		Linia->QuadratLinia.x1 = V1.X; 
		Linia->QuadratLinia.x2 = V0.X;
	}
	
	if(V0.Z < V1.Z) 
	{
		Linia->QuadratLinia.z1 = V0.Z;
		Linia->QuadratLinia.z2 = V1.Z;
	}
	else 
	{
		Linia->QuadratLinia.z1 = V1.Z; 
		Linia->QuadratLinia.z2 = V0.Z;
	}

	
	// { Pre clcul dels parametres de la recta (m1 i b1)}
	
	IncrementZ = V0.Z - V1.Z;
	IncrementX = V0.X - V1.X;

	

	if(IncrementX != 0 && IncrementZ != 0) //{ Calculem m1 i b1 }
	{
		
		Linia->m1 = DivFx(IncrementZ,IncrementX); // {m1}

		Linia->b1 = -MulFx(Linia->m1,V0.X) + V0.Z; // {b1}

		alpha = atan(FIXED_TO_DOUBLE(V0.Z - V1.Z)/FIXED_TO_DOUBLE(V0.X-V1.X));

		Quadrant = QuadrantPunt(-IncrementX, -IncrementZ);
		
		if(Quadrant == 2 || Quadrant == 3) alpha += PI;
		if(Quadrant == 4)   alpha += 2*PI;

		Linia->sinus_reciproc = DOUBLE_TO_FIXED((1.0/sin(alpha)));

	}
	else
	{
			if(IncrementX == 0) Linia->m1 = MAX_FX;
			else Linia->m1 = 0;

			Linia->b1 = 0;
			Linia->sinus_reciproc = 0;
	}

	Linia->distancia = DOUBLE_TO_FIXED(sqrt(pow(FIXED_TO_DOUBLE(V0.X - V1.X),2) + pow(FIXED_TO_DOUBLE(V0.Z - V1.Z),2)));

	// Per defecte cada lnia s pinta amb efecte de illuminaci...
    //SET_BIT(Linia->Propietats_Mapeig, BIT_RENDER_WITH_BRIGHT);
    Linia->Propietats_Mapeig = Propietats;


    // Configuraci de pendents...
    Linia->m_XY_sostre = ((V1.X-V0.X) == 0) ? 0: DivFx(V1.Y_SUP-V0.Y_SUP,V1.X-V0.X);
    Linia->m_ZY_sostre = ((V1.Z-V0.Z) == 0) ? 0:DivFx(V1.Y_SUP-V0.Y_SUP,V1.Z-V0.Z);

    Linia->m_XY_terra  = ((V1.X-V0.X) == 0) ? 0:DivFx(V1.Y_INF-V0.Y_INF,V1.X-V0.X);
    Linia->m_ZY_terra  = ((V1.Z-V0.Z) == 0) ? 0:DivFx(V1.Y_INF-V0.Y_INF,V1.Z-V0.Z);


    //         
    
    // ... i ja est. Ja tenim tota la informaci de linia inicialitzada.




}
#endif

///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
////
//// Funci que retorna cert si donat un punt de la recta ms el seu pendent, aquest intersecta la Linia.
////


int Linia_PuntPertanyAQuadrat(const tPunt_Fixa_2D *Punt,const tLinia *Linia)
{
	
	
	return (((Linia->QuadratLinia.x1) <= Punt->X) && (Punt->X <= (Linia->QuadratLinia.x2))  && 
		    ((Linia->QuadratLinia.z1) <=  Punt->Z) && (Punt->Z <= (Linia->QuadratLinia.z2)) );
}


////
////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////


/*#ifndef __RENDER_RECURSIU__
FUNCIO_A_MEMORIA_INTERNA
#endif*/
int Interseccio_Raig_Linia(FIXED32 Pendent_Raig, const tPunt_Fixa_2D *V0, const tPunt_Fixa_2D *V1, tPunt_Fixa_2D *Interseccio)
{
	////
	//// PRE: Els vertexs de linia estan respecte l'observador(ull) i la aritmetica formatejada a 24.8.
	////      Pendent raig esta en APF 16.16.
    ////
    ////
	////  Espai d'ull:
	////
	////		+-----------------> Z+
	////		|
	////	    |
	////		|
	////        |
	////        |
	////        |
	////    X+ \|/
	////
	////

	FIXED32 m2,b1,m1,m_diff, x_diff, z_diff;
	tPunt_Fixa_2D _V0, _V1;

	_V0.X = V0->X;
	_V0.Z = V0->Z;

	_V1.X = V1->X;
	_V1.Z = V1->Z;


	 // Obtenim el pendent de la lnia...

	x_diff = (_V1.X - _V0.X);
	z_diff = (_V1.Z - _V0.Z);

	if(x_diff == 0) { m1 = 0; }
	else
	{
	  	 if(ABS_FX(z_diff) < ONE_FX) // El denominador amplifica...
		 {
			 if(ABS_FX(x_diff) > FIXED_MUL_256(ABS_FX(z_diff)))	// Es pot considerar pendent molt gran com perqu sigui una lnia vertical...	 
			 {
				 	 _V0.Z = _V1.Z;
					 m1 = MAX_FX;
			 }
			 else
			 {
#ifdef __ARITMETICA_FIXA__	 			  		
				  m1 = DivFx_16_16(x_diff << 8,z_diff << 8);
#else
			      m1 = DivFx(x_diff, z_diff);
#endif

				  if(ABS_FX(m1) > (INT_TO_FIXED_16_16(256))) // Es pot considerar pendent molt gran com perqu sigui una lnia vertical...	 
				  {
						_V0.Z = _V1.Z;
						m1 = MAX_FX; 
				  }
			 }
				
		 }
		 else
		 {
#ifdef __ARITMETICA_FIXA__
				  m1 = DivFx_16_16(x_diff << 8,z_diff << 8);
#else
                  m1 = DivFx(x_diff,z_diff);
#endif
				  if(ABS_FX(m1) > (INT_TO_FIXED_16_16(256))) // Es pot considerar pendent molt gran com perqu sigui una lnia vertical...	 
				  {
						_V0.Z = _V1.Z;
						m1 = MAX_FX; 
				  }
		 
		 }
	}


//////////////////////// Clcul d'intersecci ///////////////////////////////////
		

		if(_V0.X == _V1.X) // {s una Linia horitzontal.}
		 {
			Interseccio->X = _V0.X;
			Interseccio->Z = FIXED_DIV_256(DivFx_16_16(_V0.X << 8,Pendent_Raig));

		 }
		 else
		 {
			if(_V0.Z == _V1.Z) // {s una Linia vertical.}
			{

				Interseccio->X = MulFx_16_16(_V0.Z,Pendent_Raig);
				Interseccio->Z = _V0.Z;
					


			}
			else // Tenim una paret no ortogonal. 8)
			{
						
						 // Obtenim el pendent del raig...
						 m2 = Pendent_Raig;

#ifdef __ARITMETICA_FIXA__						 
						 b1 = -MulFx_16_16(_V0.Z,m1) + _V0.X;
#else
						b1 = -MulFx(_V0.Z,m1) + _V0.X;
#endif
						
						 m_diff = (m2 - m1);

						 if(m_diff == 0)  // Cas especial...
							 return (b1 == 0) ? FALS : CERT;

						 
						 
						 if(ABS_FX(b1) > FIXED_MUL_256(ONE_FX)) // Overflow
						 {
							Interseccio->Z = DivFx_16_16(b1,m_diff);
						 }
						 /*else
						 {
							 if(ABS_FX(m_diff) < (ONE_FX))
								Interseccio->Z = FIXED_MUL_64(DivFx(b1,FIXED_MUL_64(m_diff)));*/
							 else
								Interseccio->Z = FIXED_DIV_256(DivFx_16_16(b1 << 8,m_diff));
						 
			 		  
#ifdef __ARITMETICA_FIXA__
						 Interseccio->X = (b1) + MulFx_16_16(Interseccio->Z,m1);
#else
						 Interseccio->X = (b1) + MulFx(Interseccio->Z,m1);
#endif

						 //{ I a partir d' aqui ja tenim les coordenades de la intersecci}

						 			
			}
		 }


					 
	// Interseccio Ok!
	return CERT;

}




int LINIA_PuntIntersectaLinia(const tPunt_Fixa_2D *Punt,const tLinia *Linia)
{
	FIXED32 Z;
	
	if(Linia_PuntPertanyAQuadrat(Punt,Linia)) // Tractem punt...
	{
		if((Linia->V0.X == Linia->V1.X) || (Linia->V0.Z == Linia->V1.Z)) // Linia vertical o horitzontal
			return CERT;
		else // Linia amb pendent
		{
			Z = MulFx(Linia->m1,Punt->X) + Linia->b1;

			if(FIXED_TO_INT(ABS_FX(Z - Punt->Z)) == 0) return CERT;
		}
	}
	
	return FALS;
}


FIXED32 Calcula_CoordenadaX(tPunt_Fixa_2D *V0,tPunt_Fixa_2D *V1, FIXED32 Zi)
{
	//
	// PRE: La entrada dels vertexs estan formatejades en aritmetica 24:8, igual que Zi.
	//
	//
	// Coordenades XZ a l'espai d'ull.
	//
	//    +-------------> Z+
	//    |
	//    |
	//    |
	//    |
	//    |
	//   \|/ 
	// X+
	//
	//


	FIXED32 Diff_X, Diff_Z;


	Diff_X = V1->X - V0->X;
	Diff_Z = V1->Z - V0->Z;

	if(Diff_X == 0) // La linia es horitzontal !
	{
		return 0;	
	}
	else
	{

		if(Diff_Z == 0) // La linia es verical --> excepci !
		{
			return 0;	
		}
		else
		{
			return DivFx(MulFx(Zi,Diff_X),Diff_Z);
		}
	}

	//
	// POST: Et calcula la coordenada Xi.
	//


}


///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////


