//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#ifndef __Linia_H
#define __Linia_H

////////////////////////////////////////////////////////////////////////////////////////////////
//																							  //	
//  Arxiu Linia.h																			  //
//  _____________																			  //	
//																							  //	
// - Aquest arxiu pertany al projecte de Llenament Avanat de Rajos (Advance Ray Casting)	  //	
//   per la Game Boy Advance.																  //			
//																							  //	
//   Projectista:																		      // 
//																							  //	
//       Jordi Espada Brau.	(E@Mail: u1034294@correu.udg.es)								  // 
//																							  //	
//   Professor del projectista:																  //	
//																							  //	
//       Gustavo Patow Ariel.	(E@Mail: dagush@ima.udg.es)    							      //	
//																							  //	
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////
//									  //
// INCLUSI DE CAPALERES NECESSARIES //
// __________________________________ //
//									  //
////////////////////////////////////////


#include "Definicions.h"
#include "Tipus.h"
#include "math.h"
#include "Mates.h"
#include "memoria.h"



/************\
| DEFINCIONS |
\************/



/********************\
| DEFINCIONS DE TIPUS|
\********************/







 /////////////////////
//	 			   //
// LA CLASSE Linia //
// _______________ //
//				   //
/////////////////////


typedef struct _tLinia{

	u32 		 ID_Linia;


	tVertex 	V0,V1;					// { Els dos punts que defineixen la lnia }
										
	tQuadrat_Fixa_2D	QuadratLinia;	// { Quadrat englobant de la lnia }
										

	FIXED32		m1,					    // { Pendent XZ de la linia de la recta.}
				b1,					    // { Desplaament al origen de ordenades.}
			    sinus_reciproc,			// { Pre calcul del sinus reciproc, per evitar el clcul d'arrel quadrada alhora de obtenir la component u.}
				distancia,              // { Distncia total de paret }

				m_XY_sostre,			// { pendent XY sostre }
				m_ZY_sostre,			// { pendent ZY sostre }

				m_XY_terra,				// { pendent XY terra }
				m_ZY_terra;				// { pendent ZY terra }


	//struct _tLinia *ProximaLinia;        // {Propera linia conectant }
    u32         Propietats_Mapeig;             // Propietats de mapeig de linia.
	
}tLinia;




	////////////
	//		  //
	// MACROS //
	// ______ //
	//		  //
	////////////


	// Macro que retorna cert si el punt donat pertany al quadrat que compren la Linia. (funci rpida)!

//#define ALTURA_ESPAI_ULL_XZ(xi,zi,pendent_ull_x,pendent_ull_z,v0_ull_ptr,y_inicial) ((y_inicial) + MulFx(pendent_ull_x,((xi)-(v0_ull_ptr)->X)) + MulFx(pendent_ull_z,((zi)-(v0_ull_ptr)->Z)))
//#define ALTURA_TERRA_ESPAI_ULL_XZ(xi,zi,pendent_ull_x,pendent_ull_z,v0_ptr)  ((v0_ptr)->Y_INF + MulFx(pendent_ull_x,((xi)-(v0_ptr)->X)) + MulFx(pendent_ull_z,((zi)-(v0_ptr)->Z)))

#define ALTURA_SOSTRE_MON_XZ(xi,zi,Linia) (Linia.V0.Y_SUP + MulFx(Linia.m_XY_sostre,((xi)-Linia.V0.X)) + MulFx(Linia.m_ZY_sostre,((zi)-Linia.V0.Z)))
#define ALTURA_TERRA_MON_XZ(xi,zi,Linia)  (Linia.V0.Y_INF + MulFx(Linia.m_XY_terra, ((xi)-Linia.V0.X)) + MulFx(Linia.m_ZY_terra, ((zi)-Linia.V0.Z)))



	// Macro que determina la distncia diagonal d' un punt donat respecte el primer vertex de la Linia.

	
	//////////////
	//		    //
	// FUNCIONS //
	// ________ //
	//		    //
	//////////////

	
	// Constructor per defecte.
	

	int Linia_PuntPertanyAQuadrat(const tPunt_Fixa_2D *Punt,const tLinia *Linia);
	
	// Acci que ens retorna l' intersecci entre la coordenada de partida + l' angle d' una recta donada (el raig)
	// l' actual recta.
	void LINIA_InicialitzarLinia(int ID_Linia,tVertex VertexInicial, tVertex VertexFinal,u32 Propietats, tLinia *Linia);

	int LINIA_PuntIntersectaLinia(const tPunt_Fixa_2D *Punt,const tLinia *Linia);

	FIXED32 Calcula_CoordenadaX(tPunt_Fixa_2D *V0,tPunt_Fixa_2D *V1, FIXED32 Zi);

int Interseccio_Raig_Linia(FIXED32 Pendent_Raig_XZ, const tPunt_Fixa_2D *V0 , const tPunt_Fixa_2D *V1, tPunt_Fixa_2D *Interseccio)
/*#ifndef __RENDER_RECURSIU__
FUNCIO_A_MEMORIA_INTERNA
#endif*/
;


	







#endif
