//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#include "Llista_Estatica.h"

//
// CREACI I INICIALITZACI LLISTA ESTATICA
//
void LE_InitLlista(int MaxElements,tLlistaEstatica *LlistaEstatica)
{

		LlistaEstatica->MaxElements = MaxElements;

		Reservar_Memoria(LlistaEstatica->InfoLlistaEstatica,void *, MaxElements);


		if(LlistaEstatica->InfoLlistaEstatica == NULL) 
		{
			PRINT_ERROR("\nError!!! ---> MEMORIA INSUFiCIENT",0);
			return;
		}

		// Inicialitzaci...
		MemSet(LlistaEstatica->InfoLlistaEstatica,0,MaxElements*sizeof(void *));
		LlistaEstatica->NElements = 0;



}

void LE_Guardar(int ID_Element,void *Info,tLlistaEstatica *Llista)
{
	void *ptr;

	if((ID_Element < (Llista)->MaxElements) && (ID_Element >= 0)) // vlid per actualitzar info...
	{
		ptr = (Llista)->InfoLlistaEstatica[ID_Element];

		if(ptr != NULL) // Alliberar info per actualitzar...
		{
			Alliberar_Punter(ptr);
		}

		
		Llista->InfoLlistaEstatica[ID_Element] = Info;
		Llista->NElements++;
		

	}
	else
	{
		PRINT_ERROR("\nError, ID fora els intervals de la llista estatica!!!",0);
	}
}

//
// EXTRACCIO INFO...
//
void *LE_Extraure(int ID_Index,tLlistaEstatica *Llista)
{
	void *ptr;
	
	if(ID_Index < 0 || ID_Index >= Llista->MaxElements) {PRINT_ERROR("\nError, ID fora els intervals de la llista estatica!!!",0);return NULL;}

	ptr = Llista->InfoLlistaEstatica[ID_Index];

	if((Llista)->NElements > 0)
	{
		if(ptr != NULL)
		{
			Llista->InfoLlistaEstatica[ID_Index] = NULL;
			Llista->NElements--;
			return ptr;
		}
		
	}

	return NULL;
}


//
// BORRAT ELEMENT...
//
void LE_Borrar(int ID_Index,tLlistaEstatica *Llista)
{
	void *ptr;

	if(ID_Index < 0 || ID_Index >= Llista->MaxElements) {PRINT_ERROR("\nError, ID fora els intervals de la llista estatica!!!",0);return;}
	
	ptr = Llista->InfoLlistaEstatica[ID_Index];

	if((Llista)->NElements > 0)
	{
		if(ptr != NULL)
		{
			Alliberar_Punter(ptr);
			Llista->InfoLlistaEstatica[ID_Index]=NULL;
			Llista->NElements--;
		}

	}
}

//
// BORRAT LLISTA
//
void LE_BorrarLlista(tLlistaEstatica *LE)
{
	int i;
	void *ptr;

	if(LE != NULL)
	{
		if(LE->InfoLlistaEstatica != NULL)
		{

			for( i = 0; i < LE->MaxElements;i++)
			{
				if((ptr = LE->InfoLlistaEstatica[i])) Alliberar_Punter(ptr);			
			}

			Alliberar_Punter(LE->InfoLlistaEstatica);
		}

		//Alliberar_Punter(LE);
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////



void TE_InitTaula(int Max_Y,int Max_X,tTaulaEstatica *TE)
{
	int i,j;
//	tLlistaEstatica *LlistaEstatica;



		TE->Max_X = Max_X;
		TE->Elements_X = 0;
//		TE->Max_Y = Max_Y;

		Reservar_Memoria(TE->LlistaEstaticaX,tLlistaEstaticaX, Max_X);
		if(TE->LlistaEstaticaX == NULL) {PRINT_ERROR("\nError!!! ---> MEMORIA INSUFiCIENT",0);	return;}

		// Inicialitzaci...
		for(j = 0; j < Max_X; j++)
		{
			TE->LlistaEstaticaX[j].Max_Y = Max_Y;
			TE->LlistaEstaticaX[j].Elements_Y = 0;
			Reservar_Memoria(TE->LlistaEstaticaX[j].InfoLlistaEstaticaY,void *, Max_Y);
			if(TE->LlistaEstaticaX[j].InfoLlistaEstaticaY == NULL) {PRINT_ERROR("\nError!!! ---> MEMORIA INSUFiCIENT",0);	return;}

			for(i = 0; i < Max_Y ; i++)
			{
				TE->LlistaEstaticaX[j].InfoLlistaEstaticaY[i] = NULL;
			}

		}



}

void TE_Guardar(int id_x,int id_y,void *Info,tTaulaEstatica *Taula)
{
	tLlistaEstaticaX *ptr_llista;
	
	if((id_x < (Taula)->Max_X) && (id_x >= 0))
	{
		ptr_llista = &(Taula)->LlistaEstaticaX[id_x];

		if((id_y >= ptr_llista->Max_Y) || (id_y < 0)) {PRINT_ERROR("\nError!!! ---> MEMORIA INSUFiCIENT",0);	return;}

		

		if(ptr_llista->InfoLlistaEstaticaY[id_y] != NULL) // Alliberar info per actualitzar...
		{
			Alliberar_Punter(ptr_llista->InfoLlistaEstaticaY[id_y]);
		}

		//ptr_llista->InfoLlistaEstaticaY[
		
		ptr_llista->InfoLlistaEstaticaY[id_y] = Info;
		ptr_llista->Elements_Y++;
		

	}
	else
	{
		PRINT_ERROR("\nError, ID fora els intervals de la llista estatica2!!!",0);
	}

}


void *TE_Extraure(int id_x,int id_y,tTaulaEstatica *Taula)
{
	tLlistaEstaticaX *ptr_llista;
	void *ptr;
	
	if(id_x < 0 || id_x >= Taula->Max_X) {PRINT_ERROR("\nError, ID fora els intervals de la llista estatica2!!!",0);return NULL;}

	ptr_llista = &Taula->LlistaEstaticaX[id_x];

	if((ptr_llista)->Elements_Y > 0)
	{
		ptr = ptr_llista->InfoLlistaEstaticaY[id_y];

		if(ptr != NULL)
		{
			ptr_llista->InfoLlistaEstaticaY[id_y] = NULL;
			ptr_llista->Elements_Y--;
			return ptr;
		}
		
	}

	return NULL;

}




void TE_Borrar(int id_x,int id_y,tTaulaEstatica *Taula)
{
	tLlistaEstaticaX *ptr_llista;
	void *ptr;

	if(id_x < 0 || id_x >= Taula->Max_X) {PRINT_ERROR("\nError, ID fora els intervals de la llista estatica2!!!",0);return;}
	
	ptr_llista = &Taula->LlistaEstaticaX[id_x];

	if((ptr_llista)->Elements_Y > 0)
	{
		if(id_y < 0 || id_y >= ptr_llista->Max_Y)  {PRINT_ERROR("\nError, ID fora els intervals de la llista estatica2!!!",0);return;}

		ptr = ptr_llista->InfoLlistaEstaticaY[id_y];

		if(ptr != NULL)
		{
			Alliberar_Punter(ptr);
			ptr_llista->InfoLlistaEstaticaY[id_y]=NULL;
			ptr_llista->Elements_Y--;
		}

	}

}

//
// BORRAT LLISTA
//
void TE_BorrarTaula(tTaulaEstatica *Taula)
{
	int i,j;
	tLlistaEstaticaX *ptr_llista;

	if(Taula != NULL)
	{
		if(Taula->LlistaEstaticaX != NULL) // 1era dimensi. ok!
		{
			for(i = 0; i < Taula->Max_X; i++)
			{
				ptr_llista = &Taula->LlistaEstaticaX[i];

				for(j = 0; j < ptr_llista->Elements_Y; j++) 
				{
					if(ptr_llista->InfoLlistaEstaticaY[j]!= NULL)
						Alliberar_Punter(ptr_llista->InfoLlistaEstaticaY[j]);
				}

				Alliberar_Punter(ptr_llista->InfoLlistaEstaticaY);
			}

			Alliberar_Punter(Taula->LlistaEstaticaX);
		}

		Alliberar_Punter(Taula);
	}
}
