//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#include "Mon.h"

#ifndef __MAPA_H
 #define __MAPA_H


#include "Tipus.h"
#include "Grafics.h"
#include "Textura.h"
#include "Sector.h"


#include "Generated_map.h"


/////////////////
// DEFINICIONS //
/////////////////

// Definici cubica mxima del mn (cap vrtex/objecte) pot sobrepassar aquests lmits...
#define DIMENSIO_MINIMA_MON_X	-16384
#define DIMENSIO_MINIMA_MON_Y	-16384
#define DIMENSIO_MINIMA_MON_Z	-16384


#define DIMENSIO_MAXIMA_MON_X	+16384
#define DIMENSIO_MAXIMA_MON_Y	+16384
#define DIMENSIO_MAXIMA_MON_Z	+16384


#define LOG2_QUADRICULA_X		9		
#define LOG2_QUADRICULA_Z		9	

#define DIMENSIO_QUADRICULA_Z	((DIMENSIO_MAXIMA_MON_X - DIMENSIO_MINIMA_MON_X) >> LOG2_QUADRICULA_X)
#define DIMENSIO_QUADRICULA_X	((DIMENSIO_MAXIMA_MON_Z - DIMENSIO_MINIMA_MON_Z) >> LOG2_QUADRICULA_Z)
#define DIMENSIO_QUADRAT_QUADRICULA_Z	(1 << LOG2_QUADRICULA_Z)
#define DIMENSIO_QUADRAT_QUADRICULA_X	(1 << LOG2_QUADRICULA_X)



#define COORDENADA_X_QUADRAT_QUADRICULA(x) ((FIXED_TO_INT(x) - DIMENSIO_MINIMA_MON_X) >> LOG2_QUADRICULA_X)
#define COORDENADA_Z_QUADRAT_QUADRICULA(z) ((FIXED_TO_INT(z) - DIMENSIO_MINIMA_MON_Z) >> LOG2_QUADRICULA_Z)

// Suport per fer cerques ms rpides en un mapa 2D. Per ocupa molt espai.
//#define __SUPORT_CERCA_RAPIDA__


#define NUMERO_SECTORS	26
#define MAX_SECTORS_MAPA NUMERO_SECTORS
#define MAX_NUMERO_SECTORS_MAPA2D  NUMERO_SECTORS

typedef struct{
	s16       Llista_Sectors_Intersectants[MAX_SECTORS_MAPA];
	s16		  NSectorsIntersectants;	
}tQuadricula_Sectors_Intersectants;


typedef struct _tMapa{

	// Estructura bsica de mapa 2D. 
	// IMPORTANT!!! TENIR CURA ALHORA DE AFEGIR/BORRAR o ORDENAR ELS SEGUENTS CAMPS!!!
	// LES MODIFICACIONS AFECTARAN AL PREGUARDAT DE MON !!!! VEURE FUNCI "MON_CrearMapaMonEnFormatCru" 
	// DEL MODUL MON.C.

	s16		ID_Mapa,
			Min_Altura, 
			Max_Altura,
			NSectors;

#ifdef __SUPORT_CERCA_RAPIDA__ // Cerca rpida	
// La quadricula es far servir alhora
// de cercar un punt en un sector de manera ms rpida.
// Cada quadrat de la quadricula contemplar la llista de sectors
// on els seu quadrat englobant hi intersecta.
// Si les dimensions de quadricula sn petites la cerca ser ms rpida, 
// per hi tindrem un consum alt de memria, perqu hi tindrem un conjunt petit de sectors intersectants
// Si la quadricula es gran, ser al revs, es adir, menys consum de
// memria, per ms lent alhora de fer la cerca, perqu hi tindrem ms sectors intersectants.
	tQuadricula_Sectors_Intersectants Quadricula_Sectors_Intersectants[DIMENSIO_QUADRICULA_Z][DIMENSIO_QUADRICULA_X]; // Llista de sector que pertanyen a la quadricula alineada desde MIN_MAPA a MAX_MAPA de longitud fixa
#endif

	tSector Sector[NUMERO_SECTORS];

}tMapa,tMapa2D;



//extern tMapaMon *MapaMon;


///////////////////////
// VARIABLES GLOBALS //
///////////////////////

//extern tSector *Sector;

/////////////////
// PROTOTIPUS //
////////////////

//void MAPA_Demo();
#ifdef __SUPORT_CERCA_RAPIDA__
void MAPA_CrearQuadricula(tMapa2D *Mapa2D);
#endif

void MAPA_CrearNouMapa(short ID_MAPA,tMapa2D *Mapa);
void MAPA_AfegirSector(tSector Sector, tMapa2D *ED_Mapa);
void MAPA_InterconectarPortalsParet(tMapa2D *ED_Mapa);
void MAPA_InterconectarPortalsSostreTerra(tMapa2D *ED_Mapa1,tMapa2D *ED_Mapa2);



#endif
