//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA



////////////////////////////////////////
//									  //
// INCLUSI DE CAPALERES NECESSARIES //
// __________________________________ //
//									  //
////////////////////////////////////////


#include "Mapa.h"
#include "geometria.h"
#include "Animacio.h"
//#include "Generated_map.h"




/////////////////////////////////////////
//									   //
// IMPLEMENTACI D' ACCIONS I FUNCIONS //
// ___________________________________ //
//									   //
/////////////////////////////////////////






//////////////////////////////////////////////////
//////////////////////////////////////////////////
//
// Constructor de mapa...
//
//

void MAPA_CrearNouMapa(short ID_Mapa,tMapa2D *Mapa)
{
	Mapa->ID_Mapa = ID_Mapa;
	Mapa->Min_Altura = DIMENSIO_MAXIMA_MON_Y;
	Mapa->Max_Altura = DIMENSIO_MINIMA_MON_Y;
	Mapa->NSectors = 0;
}



void MAPA_AfegirSector(tSector Sector, tMapa2D *Mapa)
{
	int numero_sector;

	if(Mapa->NSectors >= MAX_NUMERO_SECTORS_MAPA2D)
	{
#ifdef __SDL__
		fprintf(stderr,"\n S'ha rebassat el numero mxim de sectors a afegir.");
#endif
	}
	else
	{
		numero_sector = Mapa->NSectors;

		Mapa->Sector[numero_sector] = Sector;
		Mapa->Sector[numero_sector].ID_Sector = numero_sector;
		
		// Actualitzem les dimensions d'altura que te l'actual mapa...

		Mapa->Max_Altura = max(Mapa->Max_Altura,Sector.AlturaSostre);
		Mapa->Min_Altura = min(Mapa->Min_Altura,Sector.AlturaTerra);

		// Actualitzem el contador del numero de sectors afegit...
		Mapa->NSectors++;	
	}
}




short MAPA_EnllistarSectorsIntersectantsQuadratQuadricula(short n_sector, short n_sector_intersectant,const tMapa2D *Mapa2D, const tQuadrat_XZ *Quadrat_quadricula,s16 *Llista_Sectors_Intersectants)
{
	int			quadricula_intersecta_sector = FALS;

	if(n_sector_intersectant >= MAX_SECTORS_MAPA)
	{
		fprintf(stderr,"\n S'ha superat el mxim sectors de mapa. Assegurat que els vectors \"tQuadricula_Sectors_Intersectants.Llista_Sectors_Intersectants\" i \"tMapa.Sector\" tenen la longitud de MAX_SECTORS_MAPA = %i",MAX_SECTORS_MAPA);
		return n_sector_intersectant;
	}
	else
	{
		
		
		if((n_sector >= Mapa2D->NSectors) ) // Hem arribat al fons de la recursi! Reservem memoria per els sectors que s'ha trobat que hi intersectaven...
		{
			return n_sector_intersectant;
		}
		else
		{
			if(InterseccioQuadrats(Mapa2D->Sector[n_sector].QuadratSectorXZ,*Quadrat_quadricula)) 
			{
				quadricula_intersecta_sector = TRUE;
				Llista_Sectors_Intersectants[n_sector_intersectant] = n_sector;
				n_sector_intersectant++;
			}
			
			
			return MAPA_EnllistarSectorsIntersectantsQuadratQuadricula(
				n_sector+1, 
				n_sector_intersectant,
				Mapa2D, 
				Quadrat_quadricula, 
				Llista_Sectors_Intersectants);
			
			
		}
	}

// Retorna el numero de sectors intersectats. ID's s'han guardat a la taula. 
}

#ifdef __SUPORT_CERCA_RAPIDA__
void MAPA_CrearQuadricula(tMapa2D *Mapa)
{
	short z_actual, x_actual,/*x_quadricula,z_quadricula*/n_sectors_intersectants, n_x, n_z;
	tQuadrat_XZ Quadrat_quadricula;
	
	

	// Inicialitzaci de la quadricula...
	for( z_actual = DIMENSIO_MINIMA_MON_Z; z_actual < DIMENSIO_MAXIMA_MON_Z; z_actual+=DIMENSIO_QUADRAT_QUADRICULA_Z)
		for( x_actual = DIMENSIO_MINIMA_MON_X; x_actual < DIMENSIO_MAXIMA_MON_X; x_actual+=DIMENSIO_QUADRAT_QUADRICULA_X)
		{
			
			
			Quadrat_quadricula.x1 = x_actual;
			Quadrat_quadricula.x2 = x_actual+DIMENSIO_QUADRAT_QUADRICULA_X-1;
			Quadrat_quadricula.z1 = z_actual;
			Quadrat_quadricula.z2 = z_actual+DIMENSIO_QUADRAT_QUADRICULA_Z-1;
			
			
			// Reserva dels n sectors recursivament...

			n_sectors_intersectants = 0;

			n_z = (z_actual - DIMENSIO_MINIMA_MON_Z) >> LOG2_QUADRICULA_Z;
			n_x = (x_actual - DIMENSIO_MINIMA_MON_X) >> LOG2_QUADRICULA_X;

			
			// Netegem...

			memset(&Mapa->Quadricula_Sectors_Intersectants[n_z][n_x],0,sizeof(tQuadricula_Sectors_Intersectants));


			Mapa->Quadricula_Sectors_Intersectants[n_z][n_x].NSectorsIntersectants = MAPA_EnllistarSectorsIntersectantsQuadratQuadricula(0,0,Mapa,&Quadrat_quadricula,Mapa->Quadricula_Sectors_Intersectants[n_z][n_x].Llista_Sectors_Intersectants);

					
		}
		
}
#endif

void MAPA_InterconectarPortalsSostreTerra(tMapa2D *ED_Mapa1,tMapa2D *ED_Mapa2)
{
	int s_map1, s_map2,y1,y2,linia_1,linia_2,
		AlturaTerra_s_map1,AlturaSostre_s_map1,
		AlturaTerra_s_map2,AlturaSostre_s_map2,
		LiniaIntersectantTrobada,Coincidents,Quadrat_Englobants_Identics,
		ID_Mapa_sector_amb_menys_vertexs,ID_Mapa_sector_amb_mes_vertexs;
	
	tSector *Sector_Amb_Menys_Vertexs, *Sector_Amb_Mes_Vertexs;
	tPunt_Fixa_2D Vertex;
	
	
	// Comprovaci de intervals minims/maxims entre els dos mapes...
	
	y1 = min(ED_Mapa1->Max_Altura,ED_Mapa2->Max_Altura);
	y2 = max(ED_Mapa1->Min_Altura,ED_Mapa2->Min_Altura);
//	ymitg = (y1+y2) >> 1;
	
	if(/*(y1 < ymitg) && (ymitg < y2)*/ y1 != y2)
	{
#ifdef __SDL__
		printf("\n Error!, NO s'ha fet la interconexi de mapes que disposen de les segents alades:");
		printf("\n\n - Mapa1 t max. altura de %i i min. altura de %i.",ED_Mapa1->Max_Altura,ED_Mapa1->Min_Altura);
		printf("\n - Mapa2 t max. altura de %i i min. altura de %i.",ED_Mapa1->Max_Altura,ED_Mapa1->Min_Altura);
		printf("\n\n Aquests mapes intersecten en alada, o no coincideixen els intervals mnims/mxims.");
#endif
		return;
		
	}
	
	
	// Procedim a la interconecci automtica de portals terra sostre...
	for(s_map1 = 0; s_map1 < ED_Mapa1->NSectors; s_map1++)
	{
		for(s_map2 = 0; s_map2 < ED_Mapa2->NSectors; s_map2++)
		{
			// Comprobem terra del sector 1 sostre del sector 2 i a l'inversa...
			
			AlturaTerra_s_map1 = ED_Mapa1->Sector[s_map1].AlturaTerra;
			AlturaTerra_s_map2 = ED_Mapa2->Sector[s_map2].AlturaTerra;
			
			AlturaSostre_s_map1 = ED_Mapa1->Sector[s_map1].AlturaSostre;
			AlturaSostre_s_map2 = ED_Mapa2->Sector[s_map2].AlturaSostre;
			
			
			Quadrat_Englobants_Identics =	(
											(ED_Mapa1->Sector[s_map1].QuadratSectorXZ.x1) == (ED_Mapa2->Sector[s_map2].QuadratSectorXZ.x1) &&
											(ED_Mapa1->Sector[s_map1].QuadratSectorXZ.z1) == (ED_Mapa2->Sector[s_map2].QuadratSectorXZ.z1) &&
											(ED_Mapa1->Sector[s_map1].QuadratSectorXZ.x2) == (ED_Mapa2->Sector[s_map2].QuadratSectorXZ.x2) &&
											(ED_Mapa1->Sector[s_map1].QuadratSectorXZ.z2) == (ED_Mapa2->Sector[s_map2].QuadratSectorXZ.z2)
											);

				
				if(((AlturaTerra_s_map1 == AlturaSostre_s_map2)||(AlturaSostre_s_map1 == AlturaTerra_s_map2)) && Quadrat_Englobants_Identics) // Sectors amb terra/sostre coiciendents...
				{
					// Comprobaci exhaustiva del sector que siposi de ms vertexs, es trobim alienats a la figura del
					// sector amb mnys vertexs.
					
					if(ED_Mapa1->Sector[s_map1].NumeroLinies < ED_Mapa2->Sector[s_map2].NumeroLinies)
					{
						Sector_Amb_Menys_Vertexs = &ED_Mapa1->Sector[s_map1];
						ID_Mapa_sector_amb_menys_vertexs = ED_Mapa1->ID_Mapa;
						Sector_Amb_Mes_Vertexs	 = &ED_Mapa2->Sector[s_map2];
						ID_Mapa_sector_amb_mes_vertexs = ED_Mapa2->ID_Mapa;
					}
					else
					{
						Sector_Amb_Mes_Vertexs = &ED_Mapa1->Sector[s_map1];
						ID_Mapa_sector_amb_mes_vertexs = ED_Mapa1->ID_Mapa;
						Sector_Amb_Menys_Vertexs	 = &ED_Mapa2->Sector[s_map2];			
						ID_Mapa_sector_amb_menys_vertexs = ED_Mapa2->ID_Mapa;
					}
					
					// Ara realitzem la comprobaci...
					
					Coincidents = TRUE;
					
					for(linia_1 = 0; (linia_1 < Sector_Amb_Mes_Vertexs->NumeroLinies) && Coincidents;linia_1++)
					{
						Vertex.X = Sector_Amb_Mes_Vertexs->LiniaSector[linia_1].Linia.V0.X;
						Vertex.Z = Sector_Amb_Mes_Vertexs->LiniaSector[linia_1].Linia.V0.Z;
						
						LiniaIntersectantTrobada = FALSE;
						
						for(linia_2 = 0; (linia_2 < Sector_Amb_Menys_Vertexs->NumeroLinies) && !LiniaIntersectantTrobada;linia_2++)
						{
							if(LINIA_PuntIntersectaLinia(&Vertex,&Sector_Amb_Menys_Vertexs->LiniaSector[linia_2].Linia));
							{
								LiniaIntersectantTrobada = TRUE;
							}
						}
						
						if(!LiniaIntersectantTrobada) 
						{
#ifdef __SDL__
							fprintf(stderr,"\n\n Error!, El vertex (&i,%i) del sector %i (mapa %i) no ha trobat",Vertex.X,Vertex.Z,ID_Mapa_sector_amb_mes_vertexs,Sector_Amb_Mes_Vertexs->ID_Sector);
							fprintf(stderr,"\n cap linia del sector %i (mapa %i) que hi intersecti.",ID_Mapa_sector_amb_menys_vertexs,Sector_Amb_Menys_Vertexs->ID_Sector);
#endif
							
							Coincidents = FALSE;
						}
					}
					
					if(Coincidents) // Connexio de portal...
					{
						if((AlturaTerra_s_map1 == AlturaSostre_s_map2))
						{
							ED_Mapa1->Sector[s_map1].Portal_Terra  = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa2->ID_Mapa,ED_Mapa2->Sector[s_map2].ID_Sector , 0);
							ED_Mapa2->Sector[s_map2].Portal_Sostre = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa1->ID_Mapa,ED_Mapa1->Sector[s_map1].ID_Sector , 0);
						}
						else//	(AlturaSostre_s_map1 == AlturaTerra_s_map2))
						{
							ED_Mapa1->Sector[s_map1].Portal_Sostre  = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa2->ID_Mapa,ED_Mapa2->Sector[s_map2].ID_Sector , 0);
							ED_Mapa2->Sector[s_map2].Portal_Terra   = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa1->ID_Mapa,ED_Mapa1->Sector[s_map1].ID_Sector , 0);
						
						}
						
						
					}
				}
				
			
		}
		
	}
}

void MAPA_InterconectarPortalsParet(tMapa2D *ED_Mapa)
{
	//
	// Interconecci del portals paret del mapa...
	//

	s16 n_sector_actual, n_linia_sector_actual, n_sector_adjacent, n_linia_sector_adjacent,Portal_Trobat;
	tVertex V0_linia_actual,V1_linia_actual,V0_linia_adjacent,V1_linia_adjacent;

	for( n_sector_actual = 0; n_sector_actual < ED_Mapa->NSectors; n_sector_actual++) // Per tots els sectors del mapa fer...
	{
		for(n_linia_sector_actual = 0; n_linia_sector_actual < ED_Mapa->Sector[n_sector_actual].NumeroLinies; n_linia_sector_actual++)
		{
			V0_linia_actual.X = ED_Mapa->Sector[n_sector_actual].LiniaSector[n_linia_sector_actual].Linia.V0.X;
			V0_linia_actual.Z = ED_Mapa->Sector[n_sector_actual].LiniaSector[n_linia_sector_actual].Linia.V0.Z;

			V1_linia_actual.X = ED_Mapa->Sector[n_sector_actual].LiniaSector[n_linia_sector_actual].Linia.V1.X;
			V1_linia_actual.Z = ED_Mapa->Sector[n_sector_actual].LiniaSector[n_linia_sector_actual].Linia.V1.Z;
			// Cerca de la lnia per dems sectors, que tingui els vertexs inversos iguals (linia_actual.V0 = linia_sector_adjacent.V1 i linia_actual.V1 = linia_adjacent.V0)					

			for(n_sector_adjacent = 0; n_sector_adjacent < ED_Mapa->NSectors; n_sector_adjacent++)
			{
				if((n_sector_adjacent != n_sector_actual) && Punt_Pertany_A_Quadrat(V1_linia_actual.X,V1_linia_actual.Z,ED_Mapa->Sector[n_sector_actual].QuadratSector3D)) // El portal es troba aqui!!!!
				{
					Portal_Trobat = FALSE;
					
					for(n_linia_sector_adjacent = 0; (n_linia_sector_adjacent < ED_Mapa->Sector[n_sector_adjacent].NumeroLinies) && !Portal_Trobat; n_linia_sector_adjacent++)
					{
						V0_linia_adjacent.X = ED_Mapa->Sector[n_sector_adjacent].LiniaSector[n_linia_sector_adjacent].Linia.V0.X;
						V0_linia_adjacent.Z = ED_Mapa->Sector[n_sector_adjacent].LiniaSector[n_linia_sector_adjacent].Linia.V0.Z;

						V1_linia_adjacent.X = ED_Mapa->Sector[n_sector_adjacent].LiniaSector[n_linia_sector_adjacent].Linia.V1.X;
						V1_linia_adjacent.Z = ED_Mapa->Sector[n_sector_adjacent].LiniaSector[n_linia_sector_adjacent].Linia.V1.Z;

						if((V1_linia_actual.X == V0_linia_adjacent.X) && (V1_linia_actual.Z == V0_linia_adjacent.Z) && (V0_linia_actual.X == V1_linia_adjacent.X) && (V0_linia_actual.Z == V1_linia_adjacent.Z)) // Es un portal !!!
						{
							ED_Mapa->Sector[n_sector_actual].LiniaSector[n_linia_sector_actual].Portal	  = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa->ID_Mapa,n_sector_adjacent, n_linia_sector_adjacent);
							ED_Mapa->Sector[n_sector_adjacent].LiniaSector[n_linia_sector_adjacent].Portal = ENCODE_ID_MAP_SECTOR_LINE(ED_Mapa->ID_Mapa, n_sector_actual  , n_linia_sector_actual  );
							

							Portal_Trobat = TRUE;																			
						}
						
					}
				}
			
			}
		}

	}

	//
	// Interconecci del portals sostre/terra del mapa (prximament)...
	//


	
}

