//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA

#ifndef __SECTOR_H
#define __SECTOR_H

////////////////////////////////////////////////////////////////////////////////////////////////
//																							  //	
//  Arxiu SECTOR.H																			  //
//  ______________																			  //	
//																							  //	
// - Aquest arxiu pertany al projecte del motor 3D basat en un Llenament Avanat de Rajos    //
//   (Advance Ray Casting) per la Game Boy Advance.											  //			
//																							  //	
//   Projectista:																		      // 
//																							  //	
//       Jordi Espada Brau.	(E@Mail: u1034294@correu.udg.es)								  // 
//																							  //	
//   Professor del projectista:																  //	
//																							  //	
//       Gustavo Patow Ariel																  //	
//																							  //	
////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////
//					      //
// INCLUSI DE LLIBRERIES //
// ______________________ //
//					      //
////////////////////////////

#include "Linia.h"
#include "Mates.h"
#include "animacio.h"
//#include "objecte3d.h"
#include "memoria.h"

#include <stdlib.h>






////////////////////////
//					  //
// DEFINICI DE SECTOR //
// __________________ //
//					  //
////////////////////////


#define MAX_LINIES_SECTOR		20

typedef struct _tLiniaSector *PLiniaSector;

typedef struct _tLiniaSector{

	tLinia Linia; //     { Informaci de lnia }

	u32			Portal;	//   { Per apsectes de renderitzaci, Si < 0 --> Paret. Altrament s portal. Els 16 bits de ms pes representaran el ID sector i els 16 bits de menys ps el ID de lnia }


	s32         Textura_Paret;				//    { Textura, pel mapeig de tira de paret. Si < 0 -> Color Slid. Altrament mapeig de textura }

	s32			Textura_Superior; //     { Textura, pel mapeig superior de tira de portal. Si < 0 -> Color Slid. Altrament mapeig de textura }

	s32			Textura_Inferior; //     { Textura, pel mapeig inferior de tira de portal. Si < 0 -> Color Slid. Altrament mapeig de textura }


	u32			Propietats;

	u32			LiniaSector_Seguent;	// Interconnexi seguent de lnia.
	u32			LiniaSector_Anterior;	// Interconnexi anterior de lnia.

	//	
	// Propietats explica el segent:
	//
	// [X|X|X|X|X|Pe|Po]
	//  7 6 5 4 3 2 1 0
	//
	// X: Estat de bit indiferent.
	// Po: Si est activat indica que la linia s paret, altrament s portal. Util pels aspectes de jugador
	// Pe: Si est activat indica que la linia t efecte.



}tLiniaSector;


#define IS_WALL			(n)			(n&(0x1<<1))
#define SET_IS_WALL		(n)			((n)= (n) |(0x1<<1))
#define HAS_EFFECT		(n)			(n&(0x1<<2))
#define SET_HAS_EFFECT	(n)			((n)= (n) |(0x1<<2))


//////////////////////
//	 			    //
// LA CLASSE SECTOR //
// ________________ //
//				    //
//////////////////////
/***************************************\
| TIPUS DEFINICITS PER LA CLASSE OBJECTE|
\***************************************/


//typedef struct    _tLlista_Objectes_Sector_Render	*PLlista_Objectes_Sector_Render;


#ifdef __UTILITZAR_EL_MON_PREGUARDAT
#define	CONST	//const
#else
#define CONST  
#endif

typedef struct _tSector{

	// Variables constants...


	u32								ID_Sector;

	tLiniaSector					LiniaSector[MAX_LINIES_SECTOR];  // Punter a les linnes que definirn el sector actual.

		
	tQuadrat_Fixa_3D				QuadratSector3D; // Quadrat englobant 3D del sector.
	tQuadrat_XZ						QuadratSectorXZ;

//	CONST	tPoligon_4Vertexs				Pla_Terra, Pla_Sostre; // Pendent terra/sostre definit per 4 vertexs. Les coordenades XZ estan associats a les del quadrat englobant de sector.
	

/////- VARIABLES PER CONSTRUIR EL MAPA-//////////	
	s32								NumeroLinies;	// Numero de linies que definirn al sector.
/////////////////////////////////////////////////
	s32								ID_Animacio_Sostre; // { Si < 0 -> Color Slid. Altrament mapeig de textura }
	s32								ID_Animacio_Terra; //{ Si < 0 -> Color Slid. Altrament mapeig de textura }


	
	s32								Intensitat_Llum_Sector;



	// Afegit a la versi 0.953 per renderitzar pendents...
	
	//tQuadrat_Fixa_3D_4Punts				Quadrat_Altura_Sostre;//hs_x1,hs_x2,hs_z1,hs_z2;			// Altures per sostre i altures per terra, respectiu a les arestes del quadrat englobant de sector. S'ha afegit per la renderitzaci de pendents.
	//tQuadrat_Fixa_3D_4Punts				Quadrat_Altura_Terra;//ht_x1,ht_x2,ht_z1,ht_z2;         
	u32							Portal_Sostre;  // Portal del sostre...
	u32							Portal_Terra; // Portal del terra....
	

	s32	  Propietats;

	//  15 14 13 12 11 10  9  8  7  6  5  4   3    2    1    0
	//  [ ][ ][ ][ ][ ][ ][ ][ ][ ][ ][ ][ ][ ]   [SQ]  [SP] [TP]

	// Bit 1 (TP): Terra t pendent
	// Bit 2 (SP): Sostre t pendent  
	// Bit 3 (SQ): Sector quadrat
	// Bit 4     :
	//   
	//   
	//   
	// Bit 32    :
	//
	//
	//


	// Variables no-constants...

	

	s32								AlturaTerra;  // Altura de terra del sector original.
	s32								AlturaSostre; // Altura de sostre del sector original.
	



	
}tSector;
	
	// Funcions privades

//extern tSector *Sector;

typedef struct{


	u16			ID_Sector;
	s16			iluminacio; 
	s16			Textura_Terra; // IDs
	s16			Textura_Sostre; // IDs
	s16			 AlturaTerra,  AlturaSostre, NumeroDeLinies;

	// Afegit a la versi 0.953 per renderitzar pendents...
	tQuadrat_Fixa_3D				Init_Quadrat_Sector;//hs_x1,hs_x2,hs_z1,hs_z2;			// Altures per sostre i altures per terra, respectiu a les arestes del quadrat englobant de sector. S'ha afegit per la renderitzaci de pendents.

	int         Pendent_X_Sostre;
	int         Pendent_X_Terra;
		
	// ms info...
	
}tParametres_Nou_Sector;

//////////////////////
//					//
// MACROS PUBLIQUES //
// ________________ //
//					//
//////////////////////



#define Punt_Pertany_A_Quadrat_Sector(X,Z,Quadrat)\
	( ((Quadrat[0].X) <= (X) && (X) <= (Quadrat[3].X)) && \
	  ((Quadrat[0].Z) <= (Z) && (Z) <= (Quadrat[3].Z)) )


#define SET_FLOOR_WITH_SLOPE(n)		((n)= (n) |(0x1<<1))
#define SET_CEIL_WITH_SLOPE(n)		((n)= (n) |(0x1<<2))
#define SET_SQUARE_SECTOR(n)		((n)= (n) |(0x1<<3))
//#define SET_Z_CEIL_WITH_SLOPE(n)		((n)= (n) |(0x1<<4))


#define FLOOR_WITH_SLOPE(n)		(n&(0x1<<1))
#define CEIL_WITH_SLOPE(n)		(n&(0x1<<2))
#define SQUARE_SECTOR(n)		(n&(0x1<<3))



#define ENCODE_ID_MAP_SECTOR(id_map, id_sector)					(((((((id_map+1)&0xFF) << 8) | (0 & 0xFF))<< 16) ) | ((id_sector) & 0xFFFF))
#define ENCODE_ID_MAP_SECTOR_LINE(id_map, id_sector, id_line)	(((((((id_map+1)&0xFF) << 8) | (id_line & 0xFF))<< 16) ) | ((id_sector) & 0xFFFF))

#define DECODE_ID_MAP(id_encoded)				((((id_encoded) >> 24) & 0xFF)-1)
#define DECODE_ID_LINE(id_encoded)				(((id_encoded) >> 16) & 0xFF)
#define DECODE_ID_SECTOR(id_encoded)			((id_encoded) & 0xFFFF)

#define RETURN_SECTOR_POINTER(id_encoded)	    ((id_encoded) != PARET) ? &MapaMon->Mapa2D[DECODE_ID_MAP(id_encoded)].Sector[DECODE_ID_SECTOR(id_encoded)] : NULL
#define RETURN_LINE_SECTOR_POINTER(id_encoded)	((id_encoded) != PARET) ? &MapaMon->Mapa2D[DECODE_ID_MAP(id_encoded)].Sector[DECODE_ID_SECTOR(id_encoded)].LiniaSector[(DECODE_ID_LINE(id_encoded))] : NULL






	//////////////
	//		    //
	// FUNCIONS //
	// ________ //
	//		    //
	//////////////


	// Constructor amb pas de parametres.

	void Sector_NouSector(tParametres_Nou_Sector Parametres_Sector, tSector *Sector);

	int Punt_Pertany_A_Sector( FIXED32 X, FIXED32 Z, const tSector *Sector, int *intersecta_linia);// FUNCIO_A_MEMORIA_INTERNA;
	int Punt_Intersecta_Vertex_Portal(FIXED32 X, FIXED32 Z, const tSector *Sector);

	// Gestions de inserir linees i actualitzar les dimensions del sector, o sigui,el quadrat que engloba el sector.

	//void Sector_AfegirLinia(const tLinia *Linia,int NumeroDeLinia,tImatge *textura_inferior,tImatge *textura,tImatge *textura_superior,PSector Portal,tSector *Sector);
	void SECTOR_AfegirLinia( tVertex V0, tVertex V1,s16 id_textura_inferior, s16 id_textura, s16 id_textura_superior, u32 Propietats_Mapeig,tSector *Sector);
	
	tLiniaSector *Punt_Intersecta_Linia_Sector(const tPunt_Fixa_2D *punt, const tSector *Sector);



#endif

