//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#ifndef __MEMORIA_H__
#define __MEMORIA_H__
#include "definicions.h"
#include "malloc.h"

#ifdef __HAM__
#include "mygba.h"

#define FUNCIO_A_MEMORIA_INTERNA	MEM_FUNC_IN_IWRAM
#define MEM_A_MEMORIA_INTERNA		MEM_IN_IWRAM
#define MEM_A_MEMORIA_EXTERNA		MEM_IN_EWRAM
#else
#define FUNCIO_A_MEMORIA_INTERNA	
#define MEM_A_MEMORIA_INTERNA		
#define MEM_A_MEMORIA_EXTERNA		
#endif




/*********\
| V.GLOBAL|
\*********/

extern int mem_total_used;
extern int mem_max_used;

/********\
| MACROS |
\********/

#define CopyMem_8bit(ptr_dest, ptr_src, n_bytes)    memcpy((ptr_dest),(ptr_src),n_bytes);
/*#define CopyMem_16bit(ptr_dest, ptr_src, n_half_words)\
if(((n_half_words) & 0x1) == 0)  memcpy((u16 *)(ptr_dest),(u16 *)(ptr_src),(n_half_words)<< 1);\
else memcpy((u16 *)(ptr_dest),(u16 *)(ptr_src),(n_half_words) << 1);\*/

//#define CopyMem_16bit(ptr_dest, ptr_src, n_half_words)   memcpy((ptr_dest),(ptr_src),(n_half_words)*sizeof(s16));
#define SOFT_CopyMem_16bit(ptr_dest, ptr_src, n_half_words)		memcpy((ptr_dest),(ptr_src),(n_half_words)*sizeof(s16));
#define SOFT_CopyMem_32bit(ptr_dest, ptr_src, n_words)			memcpy((ptr_dest),(ptr_src),(n_words)*sizeof(s32));
#define SOFT_MemSet(ptr_dest,valor,n_bytes)						memset(ptr_dest,valor,n_bytes)
#define SOFT_MemSet16(ptr_dest,valor,n_words)\
{\
    int i = 0;\
    while(i < (n_words)) {(ptr_dest)[i++] = valor;}\
}\

#define SOFT_MemSet16X(ptr_dest,valor,n_hword)                  memset(ptr_dest,valor,n_hword)
				


#ifdef __HAM__

#define ReservarMemoriaInit ;
#define Reservar_Memoria(punter,tipus,n_elements)  (punter)=((tipus *)malloc(sizeof(tipus)*(n_elements)));
#define Alliberar_Punter(punter)  free(punter);


#define HAM_CopyMem_32bit(ptr_dest, ptr_src, n_blocks)      TOOL_DMA3_SET((ptr_src),(ptr_dest),(n_blocks),DMA_TRANSFER_32BIT,DMA_STARTAT_NOW)
#define HAM_CopyMem_16bit(ptr_dest, ptr_src, n_half_words)\
if(((n_half_words) & 0x1) == 0){TOOL_DMA3_SET((ptr_src),(ptr_dest),(n_half_words >> 1),DMA_TRANSFER_32BIT,DMA_STARTAT_NOW);}\
else{TOOL_DMA3_SET((ptr_src),(ptr_dest),(n_half_words),DMA_TRANSFER_16BIT,DMA_STARTAT_NOW);}\

extern void ARM_MemSet(void *dst,char valor, int n_bytes);
extern void ARM_MemSet16(short *dst,short valor, int n_hwords);

#define MemSet		ARM_MemSet
#define MemSet16	ARM_MemSet16

/*#define CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)\
if(((n_blocks_16bit) & 0x1) == 0) {HAM_CopyMem_32bit(ptr_dest, ptr_src, (n_blocks_16bit) >> 1)}\
else {HAM_CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)}\*/

#define CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit) HAM_CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)

  /*
if(((n_blocks_16bit) & 0x1) == 0) {SOFT_CopyMem_32bit(ptr_dest, ptr_src, (n_blocks_16bit) >> 1)}\
else {SOFT_CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)}\*/

#endif

#ifdef __SDL__

#define ReservarMemoriaInit {mem_total_used = 0;mem_max_used = 0;}

#define Reservar_Memoria(punter,tipus,n_elements)\
{(punter)=((tipus *)malloc(sizeof(tipus)*(n_elements)));\
mem_total_used+=(sizeof(tipus)*(n_elements));\
mem_max_used = max(mem_total_used, mem_max_used);}\

#define Alliberar_Punter(punter)\
{mem_total_used-=_msize((punter));\
free(punter);}\


#define SDL_CopyMem_16bit(ptr_dest, ptr_src, n_half_words)      memcpy((ptr_dest),(ptr_src),(n_half_words)*sizeof(s16));
#define SDL_CopyMem_32bit(ptr_dest, ptr_src, n_words)           memcpy((ptr_dest),(ptr_src),(n_words)*sizeof(s32));

#define CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)\
if(((n_blocks_16bit) & 0x1) == 0) {SDL_CopyMem_32bit(ptr_dest, ptr_src, (n_blocks_16bit) >> 1)}\
else {SDL_CopyMem_16bit(ptr_dest, ptr_src, n_blocks_16bit)}\

#define MemSet		SOFT_MemSet
#define MemSet16	SOFT_MemSet16

#endif



#endif
