//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#include "GFX_Grafics.h"
#include "TGA.h"






int Recursive_Generate_MipMap(tMipMap *Base_MipMap, const tImage *Original_Image, int Actual_MipMap, int Max_MipMaps)
{
	int width, height, x, y, p1, p2, p3, p4,i,pos,red_med,green_med,blue_med;
	tImage Info_MipMap;
	char cad[25],cad2[25];

	if(Actual_MipMap > Max_MipMaps || Actual_MipMap == 0) // Return we arrived in the top level Base_MipMap.
	{
		return Actual_MipMap;
	}
	else
	{
		// Reserve memory for pixels buffer, for actual Base_MipMap...
		

		// Force to multiple of 2...
		width  = Base_MipMap[Actual_MipMap-1].Width;
		height = Base_MipMap[Actual_MipMap-1].Height;
		Base_MipMap[Actual_MipMap].pixels = NULL;

		if((width & 0x1) != 0)
		{
			width++;
			return Actual_MipMap;
		}

		if((height & 0x1) != 0)
		{
			height++;
			return Actual_MipMap;
		}
	
		width = Base_MipMap[Actual_MipMap].Width  =  width  >> 1;
		height= Base_MipMap[Actual_MipMap].Height =  height >> 1;
		


		// Reserve memory for pixels buffer, for the first image...
		
		if((Base_MipMap[Actual_MipMap].pixels = (uint *) malloc(width*height*sizeof(uint))) == NULL)
		{
			fprintf(fp_err,"\n Error, cannot reserve %i bytes for pixels!",width*height);
			return FALSE;
		}

		

		
		for(y = 0; y < Base_MipMap[Actual_MipMap-1].Height; y+=2)
		{
			for(x = 0; x < Base_MipMap[Actual_MipMap-1].Width; x+=2)
			{
				
				p1 = Base_MipMap[Actual_MipMap-1].pixels[y*Base_MipMap[Actual_MipMap-1].Width+x];
				p2 = Base_MipMap[Actual_MipMap-1].pixels[y*Base_MipMap[Actual_MipMap-1].Width+x+1];
				p3 = Base_MipMap[Actual_MipMap-1].pixels[(y+1)*Base_MipMap[Actual_MipMap-1].Width+x];
				p4 = Base_MipMap[Actual_MipMap-1].pixels[(y+1)*Base_MipMap[Actual_MipMap-1].Width+x+1];

				red_med = (((p1 >> 16)&0xFF) + ((p2 >> 16)&0xFF) + ((p3 >> 16)&0xFF) + ((p4 >> 16)&0xFF)) >> 2;
				green_med = (((p1 >> 8)&0xFF) + ((p2 >> 8)&0xFF) + ((p3 >> 8)&0xFF) + ((p4 >> 8)&0xFF)) >> 2;
				blue_med = (((p1)&0xFF) + ((p2)&0xFF) + ((p3)&0xFF) + ((p4)&0xFF)) >> 2;

				Base_MipMap[Actual_MipMap].pixels[(y>>1)*width + (x >> 1)] = (red_med << 16) | (green_med << 8) | blue_med;

				

				
				

			
			}
		}
		
		Info_MipMap.pixels = Base_MipMap[Actual_MipMap].pixels;
		Info_MipMap.Height = height;
		Info_MipMap.Width = width;
		Info_MipMap.bpp = 24;
		memset(cad,0,sizeof(cad));
		memset(cad,0,sizeof(cad2));

		pos = strlen(Original_Image->name)-1;

		while((Original_Image->name[pos] != '/') && (Original_Image->name[pos] != '\\') && (pos >= 0))
		{
			pos--;
		}

		pos++;
		i = pos;

		while((Original_Image->name[i] != '.') && (i < ((int)strlen(Original_Image->name)-1)))
		{

			if(Original_Image->name[i] != '.')
				cad2[i-pos] = Original_Image->name[i];
			
			i++;
		}

		sprintf(cad,"Out\\%05s_mip%2i.tga",cad2,Actual_MipMap);

		SaveTGA(cad,&Info_MipMap);

		return Recursive_Generate_MipMap(Base_MipMap, Original_Image, Actual_MipMap+1, Max_MipMaps);
	
	
	}

	//return TRUE;
	
}



tMipMap *Generate_MipMaps(tImage *Original_Image)
{
	//
	// PRE: We must pre-copy 
	//

	tMipMap *Base_MipMap;

	// Reserve memory for N_MipMaps...

	if((Base_MipMap = (tMipMap *) malloc(sizeof(tMipMap)*(MAX_MIPMAPS+1))) == NULL)
	{
		fprintf(fp_err,"\n Error, cannot reserve %i bytes for generate the mipmaps!",sizeof(tMipMap)*(MAX_MIPMAPS+1));
		return NULL;
	}

	// Reserve memory for pixels buffer, for the first image...

	Base_MipMap[0].pixels = NULL;

	if((Base_MipMap[0].pixels = (uint *) malloc(Original_Image->Height*Original_Image->Width*sizeof(uint))) == NULL)
	{
		fprintf(fp_err,"\n Error, cannot reserve %i bytes for pixels!",Original_Image->Height*Original_Image->Width*sizeof(uint));
		return NULL;
	}

	// Copy the width, height and pixels...

	Base_MipMap[0].Height = Original_Image->Height;
	Base_MipMap[0].Width = Original_Image->Width;
	memcpy(Base_MipMap[0].pixels,Original_Image->pixels,Original_Image->Height*Original_Image->Width*sizeof(int));

	// And we'll generate N_MIPMAPS recursevily...

	if(!MASK_IMAGE_PROPIETRY(Original_Image->Propietries,IS_PANORAMA) && (MASK_IMAGE_PROPIETRY(Original_Image->Propietries,WITH_MIPMAPS)))
	{
		Original_Image->N_MipMaps = (Recursive_Generate_MipMap(Base_MipMap,Original_Image, 1,MAX_MIPMAPS)-1); // -1 because the mipmap 0 acount, too.
		

/*		{
			
			Original_Image->N_MipMaps = N_MIPMAPS+1; // +1 because the we have saved the original image onto mipmap 0...
			
		
		}
		else
		{
			return NULL;	
		}*/
	}
	else
	{
		Original_Image->N_MipMaps = 0;
	}

	return Base_MipMap;


	


}
