//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#include "TGA.h"
#include "stdlib.h"
#include "string.h"
#include "stdio.h"





int LoadTGA(char *filename, tImage *Image) // Load A TGA File into struture image...
{
	uchar TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0},
		 TGAcompare[12],
		 header[6];
	FILE	*fp_TGA;

	uint		imageSize;
	int			i,rows_processed,inc_scanline,j,Ini_Offset;
//	uchar		r,g,b;
	uchar		pixel[3], Vertical_Flip;
	


	if((fp_TGA = fopen(filename,"rb")) != NULL)
	{

		if(fread(TGAcompare,1,sizeof(TGAcompare),fp_TGA) != sizeof(TGAcompare)) EXIT
//		if(memcmp(TGAheader,TGAcompare,sizeof(TGAheader)) != 0) EXIT
		if(fread(header,1,sizeof(header),fp_TGA) != sizeof(header)) EXIT

		// Dimensions of image....

		Image->Width = (header[1] << 8) + header[0];
		Image->Height = (header[3] << 8) + header[2];

		Image->bpp = header[4];

		Vertical_Flip = ((header[5]&0x20) != 0x20);
		
		imageSize = Image->Width*Image->Height;

		// load the image...

		Image->pixels = (uint *) malloc(imageSize*sizeof(uint));

	/*	if(fread(Image->pixels,Image->bpp/8,imageSize, fp_TGA) != imageSize)
		{
			free(Image->pixels);
			fprintf(fp_err,"\n Don't enought memory for load %s.tga !!!",filename);
			EXIT;
		
		}*/

		if(Vertical_Flip) 
		{
			Ini_Offset = Image->Width*Image->Height - Image->Width;
			inc_scanline = -Image->Width;
		}
		else 
		{
			Ini_Offset = 0;
			inc_scanline = Image->Width;
		}



		
		for(i = Ini_Offset, rows_processed = 0; rows_processed < Image->Height; rows_processed++, i+=inc_scanline )
		{
			for(j = 0; j < Image->Width; j++)
			{
				if(feof(fp_TGA)) 
				{
					free(Image->pixels);
					fprintf(fp_err,"\n The image %s is more sizer than those attributes!!!",filename,imageSize);
					
					EXIT;
				}
				else
				{
					// read the triplete BGR...
					
					fread(pixel,1,Image->bpp/8,fp_TGA);
					
					/*		pixel[0] = (Image->pixels[i]&0x00FF0000) >> 16;		// Capture R
					pixel[1] = (Image->pixels[i]&0x0000FF00) >> 8;		// Capture G
					pixel[2] =  Image->pixels[i]&0x000000FF;			// Capture B*/
					
					// Save information pixel in RGB order...				
					Image->pixels[i+j] = (pixel[2] << 16)| (pixel[1] << 8)| pixel[0];
				}
			}
			
			
		}


		
	
		fclose(fp_TGA);
	
		return TRUE;
	}


	return FALSE;


}



void SaveTGA(char *filename, tImage *Image) // Save struct image to TGA File...
{
	uchar TGAheader[12]={0,0,0x2,0,0,0,0,0x18,0,0,0,0},
//	TGAcompare[12],
	header[6];
	FILE	*fp_TGA;

	uint		imageSize,i;
//	uchar		r,g,b;
	uchar       pixel[3];//,aux;


	if((fp_TGA = fopen(filename,"wb")) != NULL)
	{

		// Save TGA header...
		fwrite(TGAheader,1,sizeof(TGAheader),fp_TGA);

		// Second part of header...
		header[0] = Image->Width & 0xFF;
		header[1] = (Image->Width & 0xFF00) >> 8;

		header[2] = Image->Height & 0xFF;
		header[3] = (Image->Height & 0xFF00) >> 8;

		header[4] = Image->bpp;
		header[5] = 0;
		header[6] = 0;


		fwrite(header,1,sizeof(header),fp_TGA);

		// And finally write the pixels...
		
		


//		if(fread(TGAcompare,1,sizeof(TGAcompare),fp_TGA) != sizeof(TGAcompare)) EXIT
//		if(memcmp(TGAheader,TGAcompare,sizeof(TGAheader)) != 0) EXIT
		

		// Dimensions of image....

		Image->Width = (header[1] << 8) + header[0];
		Image->Height = (header[3] << 8) + header[2];
		imageSize = Image->Width*Image->Height;

		Image->bpp = 24;
		
			
		// inverse R/B

		for(i = 0; i < imageSize; i++)
		{

			pixel[2] = ((Image->pixels[i]) >> 16)&0x000000FF;    // Capture R
			pixel[1] = ((Image->pixels[i]) >> 8)&0x000000FF;     // Capture G
			pixel[0] =  Image->pixels[i]&0x000000FF;			// Capture B

			//Image->pixels[i] = 	Image->pixels[i] = (r << 16)| (g << 8)| b;	

			fwrite(pixel,1,Image->bpp/8,fp_TGA);
		
			
		}

		
		
	
		fclose(fp_TGA);
	
	
	}






}

