#ifndef __TYPES_H__
#define __TYPES_H__

#include "stdio.h"

/*********\
| DEFINES |
\*********/



#define MAX_SPECIAL_CHARACTERS  2
#define TRUE					1
#define FALSE					0
#define WITH_STRING				50

#define uint					unsigned int
#define ushort					unsigned short
#define uchar					unsigned char

enum{
		IS_PASSABLE = 0,        // Ex. Si textura = efecte portal vallat --> No es passable. 
                                //     Altrament si textura = efecte portal amb taranyines --> Es passable
		IS_PANORAMA,            // S' explica si la imatge representa un panorama...
		WITH_MIPMAPS,           // S' explica si la textura t mipmaps...
		WITH_BRIGHT,            // NO cal fer aquesta consulta! Es interna a propietats d 'objecte de motor.
		IS_FONT,                // S' explica si la imatge representa una font. 
        IS_SPRITE,              // L' eina GFX determina si la imatge s un sprite.
        IS_DIMX_POW2,           // L' eina GFX determina si la imatge t una amplada potencia de 2.
        IS_DIMY_POW2,           // L' eina GFX determina si la imatge t una altura potencia de 2.
};

// defines for 3D engine...
enum{
    BIT_RENDER_WITH_BRIGHT = 0,
    BIT_RENDER_WITH_GLENZ

};


#define		MASK_IS_PASSABLE	 (0x1 << IS_PASSABLE)
#define		MASK_IS_PANORAMA	 (0x1 << IS_PANORAMA)
#define		MASK_WITH_MIPMAPS	 (0x1 << WITH_MIPMAPS)
#define		MASK_WITH_BRIGHT	 (0x1 << WITH_BRIGHT)
#define		MASK_IS_FONT		 (0x1 << IS_FONT)
#define		MASK_IS_SPRITE		 (0x1 << IS_SPRITE)
#define		MASK_IS_DIMX_LOG2	 (0x1 << IS_DIMX_LOG2)
#define		MASK_IS_DIMY_LOG2	 (0x1 << IS_DIMY_LOG2)


/*********\
| MACROS  |
\*********/

#define SET_MASK_IMAGE_PROPIETRY(p,id)  ((p) |= ((0x1) << id)) 
#define MASK_IMAGE_PROPIETRY(p,id)      ((p&(0x1 << (id))) != 0)

/**********\
| TYPEDEFS |
\**********/



typedef struct{
	int  *pixels; // RGB mode and palette mode...
	int   Width, Height,bpp;
    char  *palette;
}tMipMap;


typedef struct{

	unsigned char name[WITH_STRING]; // name of image...
	int   *pixels; // Pixels of image...
	int   Width, Height;
	int   bpp;  // Current size of bpp...
	int   Is_Passable;
	int   Is_Panorama;
	int   Is_Font;
    int   Is_Sprite;
    int   Is_DimXPow2;
    int   Is_DimYPow2;
    int   With_MipMaps;
	int   With_Bright;

	uint  Propietries;

	tMipMap  *MipMaps; // First mipmap (enumerate as 0) is the original image, and nexts the another mipmaps...
	int      N_MipMaps; // Total possile mipmaps.
	char	 *palette; // When the image is in 8 bpp format...
	

}tImage;

typedef struct _tNodeImage{

	int   ID_Node; // Identificator node...

	tImage  Data_Type;

	struct _tNodeImage *PNext;


}tNodeImage;



typedef struct _tNodeFrame{

	int    ID_Image;
	int    ID_Frame;
	
	struct _tNodeFrame *PNextFrame;

	
}tNodeFrame;


typedef struct _tNodeMasterAnimation{

	char    name[WITH_STRING];

	int		Is_Animation;
	
	struct _tNodeFrame			 *PFirstFrame;
	struct _tNodeMasterAnimation *PNextNode;

	
}tNodeMasterAnimation;

#endif