//    Copyright (C) 2006 Jordi Espada Brau (jespa003@hotmail.com, u1034294@correu.udg.es)
//    Free Software Foundation, Inc.  Contributed by Gustawo Patow Ariel
//
//    This file is part of Jespa3D Engine.
//
//    Jespa3D Engine is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Jespa3D Engine is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Jespa3D Engine; ; see the file COPYING. if not, write to 
//    the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//    Boston, MA  02111-1307  USA


#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "GestFile.h"
#include "GestNode.h"
#include "Palette.h"
#include "TGA.h"
#include "Image.h"


//#define GFX_GENERATE_VERSION 1.00
#define GFX_GENERATE_VERSION 1.03



//
//	  _____ ________   __   _____                           _       
//	 / ____|  ____\ \ / /  / ____|                         | |      
//	| |  __| |__   \ V /  | |  __  ___ _ __   ___ _ __ __ _| |_ ___ 
//	| | |_ |  __|   > <   | | |_ |/ _ \ '_ \ / _ \ '__/ _` | __/ _ \
//	| |__| | |     / . \  | |__| |  __/ | | |  __/ | | (_| | ||  __/
//	 \_____|_|    /_/ \_\  \_____|\___|_| |_|\___|_|  \__,_|\__\___|
//  
//   _________________________________________________________________
//  [_________________________________________________________________]
//
//
//  This tool creates this files:
//
//   - GFX_Palette.pcx                     : Generates an PCX with palette reduced of the files described on GFX_Generate.cfg. This graphic
//											 only can be used for terms of palette, exclusive for tool reduce.
//
//   - GFX_InfoImages.c & GFX_InfoImages.h : Generates files type C for post initialitzation of the images integrated on code.
//
//   - reduce.cfg                          : Generates two files info for tool reduce: one file for the images type blocks & another for the images
//											 images type sprite. both files, includes the file GFX_Palette.pcx for quatitatize procs.
//
//
//  The tool has created by Jordi Espada Brau on 20-05-2005.
//
//
                                                                
/**********\
| DEFINES  |
\**********/

#define MAX_FIELDS				6
#define MAX_SPECS				5



/**********\
| TYPEDEFS |
\**********/


typedef struct _tInfoNode{

	int		ID_Node;							// For sorting...

	char Name[WITH_STRING];						// Name of file TGA.


	tImage	ImageInfo;

	int *pixels;								// pixels in RGB mode.
	int  width, height;							// Dimensions of graphic TGA.

	struct PNext *_tInfoNode; // Pointer to next node of this type.   `[]->[]->->[]

}tInfoNode;


typedef struct{
	int id;
	char name_spec[WITH_STRING];

}typdef_spec_field;

/*void a(void *caca)
{
	tcaca vaca;

	vaca.caca = caca;

}*/

/************\
| PROTOTYPES |
\************/

void MEM_Copy_buf8_to_buf16(short *table16, char *table8, int size);

/*****************\
| GLOBAL VARIABLES|
\*****************/



tInfoNode *List_Nodes_Type_Block;
tInfoNode *List_Nodes_Type_Sprite;


FILE	*fp_err    = NULL;
FILE	*fp_out    = NULL;
FILE	*fp_TGA	   = NULL;
FILE    *fp_config = NULL;
FILE    *fp_pal    = NULL;
char	*pal;
short   *pal16;//[256*256];
char    *shade_table;
short    shade_table16[256*256];

char    *glenz_table;
short    glenz_table16[256*256];

int main()
{
	char SpecialCharacters[MAX_SPECIAL_CHARACTERS] =  {'\t',' '};
	char Field_Name[MAX_FIELDS][WITH_STRING] ={"BLOCKS\0\0\0\0\0\0\0\0\0","BLOCKS_WITH_BRIGHT\0\0\0\0\0\0\0\0\0","SPRITES\0\0\0\0\0\0\0\0\0","SPRITES_WITH_BRIGHT\0\0\0\0\0\0\0\0\0","PANORAMAS\0\0\0\0\0\0\0\0\0","FONTS\0\0\0\0\0\0\0\0\0"};
	typdef_spec_field Specs[MAX_SPECS] ={
		IS_PASSABLE,"PASSABLE\0",
		IS_PANORAMA,"IS_PANORAMA\0",
		WITH_MIPMAPS,"WITH_MIPMAPS\0",
		WITH_BRIGHT,"WITH_BRIGHT\0",
		IS_FONT,"IS_FONT\0",
    }; // put more specs..

	int i,error,end,n_spec;

	char File_Name_With_Path[WITH_STRING], File_Name[WITH_STRING], Spec_Name[WITH_STRING];
	tImage Image;
	tNodeImage *list_nodes = NULL;
	
	

	if((fp_err = fopen("stderr.txt","w+")) == NULL) return -1;
	if((fp_out = fopen("stdout.txt","w+")) == NULL) { fclose(fp_err); return -1; }

	error = FALSE;

	system("cls");

	printf("  _____ ________   __   _____                           _	\n");
	printf(" / ____|  ____\\ \\ / /  / ____|                         | |		\n");
	printf("| |  __| |__   \\ V /  | |  __  ___ _ __   ___ _ __ __ _| |_ ___	\n");
	printf("| | |_ |  __|   > <   | | |_ |/ _ \\ '_ \\ / _ \\ '__/ _` | __/ _ \\\n");
	printf("| |__| | |     / . \\  | |__| |  __/ | | |  __/ | | (_| | ||  __/	\n");
	printf(" \\_____|_|    /_/ \\_\\  \\_____|\\___|_| |_|\\___|_|  \\__,_|\\__\\___|\n");
	printf(" _________________________________________________________________	\n");
	printf("[_________________________________________________________________]	\n");
	printf("\nversion: %.02f\n\n",GFX_GENERATE_VERSION);

	//a(&Image);


	if((fp_config = fopen("GFX_Generate.cfg","rt")) != NULL)
	{
		// Ok, let's to read file...

		for(i = 0; i < MAX_FIELDS && !error; i++)
		{

			if(Get_Pointer_To_Field(Field_Name[i])) // Ok, I can proces de field.
			{
				fprintf(fp_out,"\nI found the Field %s !",Field_Name[i]);

				// Getting the files of this field...
				while(!End_Field() && !error)
				{
					//memset(File_Name,0,sizeof(File_Name));

					Get_File_With_Path(File_Name_With_Path);

					if(LoadTGA(File_Name_With_Path, &Image))
					{
						
						Get_File_Name(File_Name, File_Name_With_Path);

						if(strlen(File_Name) <= (SIZE_NAME_FILES_DOS))
						{
							fprintf(fp_out,"\n\nThe file name %s succseful open",File_Name_With_Path);
							
							
							Image.With_Bright = FALSE;
							Image.With_MipMaps = FALSE;
							Image.Is_Passable = FALSE;
							Image.Is_Panorama = FALSE;
														
							

							if(i == 1 || i == 3) Image.With_Bright = TRUE;
							else Image.With_Bright = FALSE;
							if(i == 4) Image.Is_Panorama = TRUE;
							else Image.Is_Panorama = FALSE;
							if(i == 5) 
							{
								Image.Is_Font = TRUE;
							}
							else Image.Is_Font = FALSE;
							
							memset(Image.name,0,50);
							sprintf(Image.name,File_Name_With_Path,50);

							Image.Propietries = 0;

							
							end = FALSE;

							do
							{
								Next_Image_Spec(Spec_Name);
								// Search the propietry of image...

								for( n_spec = 0; n_spec < MAX_SPECS; n_spec++)
								{
									if(!strcmp(Spec_Name,Specs[n_spec].name_spec))
									{
										fprintf(fp_out,", with propietry \"%s\" ",Spec_Name,Specs[n_spec].name_spec);
										SET_MASK_IMAGE_PROPIETRY(Image.Propietries,n_spec);									
									}
								}

								
							}while(!End_Line());

							Next_Line();

							CLRLINE;
							printf("\rProcessing file: \"%s\"",Image.name);

							// Generating mipmaps of this image...

							if((Image.MipMaps = Generate_MipMaps(&Image)) == NULL)
							{
								error = TRUE;
							}
							else
							{
							
								// Add info on node...
								
								AddNodeList_Image_FIFO(Image, &list_nodes);
							}
							
							// Un load tga image...
							//UnLoadTGA(Image);
						}
						else // The file must have less that 9 characters....
						{
							fprintf(fp_err,"\nError! The file name \"%s\" has more than 8 characters",File_Name_With_Path);
							error = TRUE;
						}
						
					}
					else // I cannot open this file...
					{
						fprintf(fp_err,"\nI cannot open the file name %s",File_Name);
						error = TRUE;
					
					}
				}
			
			}
			else
			{
				fprintf(fp_err,"\nI can't search field [%s] !",Field_Name[i]);
				fclose(fp_config);
				error = TRUE;
			
			}
		}

		if(!error)
		{

			CLRLINE;
			printf("\rProcessing file: done!");
						
		// Get the palette with the best 256 colors...

			
		printf("\nBuilding palette...");
		if((pal = Build_Palette(list_nodes,TRUE)) == NULL)  error=TRUE;
		printf("\rBuilding the palette: done!\n");
		
		// Reduce the images and mipmaps to palette "pal"...

		Reduce_Images_To_256_Colors(list_nodes,pal);

		// Build the shade & intensity tables...
		printf("\nCreating shade table...");
		if((shade_table   = Create_Table_Shades(pal, 1)) == NULL) error = TRUE;
  //      MEM_Copy_buf8_to_buf16(shade_table16,shade_table,PALETTE_SIZE*MAX_LIGHT);
   		printf("\rCreating shade table: done!");

        // Build the glenzing table...
		printf("\nCreating glenzing table...");
		if((glenz_table   = PAL_Create_Table_Glenz(pal, 1)) == NULL) error = TRUE;
//        MEM_Copy_buf8_to_buf16(glenz_table16,glenz_table,PALETTE_SIZE*PALETTE_SIZE);
   		printf("\rCreating glenz table: done!");

		// Reduce palette to 16 bpp...

		printf("\nConverting 8:8:8 Palette to 5:6:5 palette...");
		if((pal16 = Reduce_Palete_24bpp_To_16bpp(pal)) == NULL) error = TRUE;
		printf("\rConverting 8:8:8 Palette to 5:6:5 palette: done!");
		

        printf("\nSetting image propietries...");
		IMAGE_SetImagePropietries(list_nodes);
	/*	fp_pal = fopen("palette.pal","wb");

		if(pal == NULL || fp_pal == NULL)
		{
			if(pal == NULL)
				fprintf(fp_err,"\nError: cannot reduce the palette !!!");
			else
				fprintf(fp_err,"\nError: cannot open \"palette.pal\" !!!");
			//exit(EXIT_FAILURE);
			
		}
		else
		{
			// write palette & shade tables to module C file...
					

			fwrite(pal,sizeof(char),768,fp_pal);
			for(i = 0; i < 768; i++) 
			{
				fputc(pal[i],fp_pal);
			}

			
			

			fp_pal = fopen("palette.pal","rb");

			for(i = 0; i < 768; i++) 
			{
				c= fgetc(fp_pal);
			}

			fclose(fp_pal);*/

			if(!WriteDataFiles(list_nodes)) 
			{
				fprintf(fp_err,"\nError on write data files!");
				error = TRUE;
			}
			
			if(!WriteInfoFiles(list_nodes)) 
			{
				fprintf(fp_err,"\nError on write information files!");
				error = TRUE;
			}

			printf("\n\nAll done :) !\n\n");

			

			// proba...
			//SaveTGA("caca.tga",&list_nodes->Data_Type);
			
			free(pal);
			free(shade_table);

			//fclose(fp_pal);
		
	//	}
		}


		
		fclose(fp_config);
		fclose(fp_err);
		fclose(fp_out);

	
	}
	else
	{
		fprintf(fp_err,"\nI can't open GFX_Generate.cfg !");
		
	}

	
	Delete_Nodes_Image(list_nodes);

	if(error)
	{
		exit(1);
	}
	else
	{
		exit(0);
	}
	

	

	
}



void MEM_Copy_buf8_to_buf16(short *table16, char *table8, int size)
{
    char color;
    char  *_ptr_scr;
    short *_ptr_dst;
    int i;

    for(i = 0, _ptr_dst = (short *)table16, _ptr_scr = table8; i < size; i++, _ptr_dst++, _ptr_scr++)
    {
                color = *_ptr_scr;
                *_ptr_dst = (short)(color | (color << 8));
    }
}