#include "mygba.h"
#include "math.h"

#include "gfx/master.pal.c"
#include "gfx/terra.raw.c"
#include "gfx/terra.map.c"



/*
----------------------------------
 Program entry point
----------------------------------
*/


#define RADIANT(x)   (float)(((x)*PI)/180.0f)

u8 newframe=0;
u8 frames=0;
u32 Xj = 256 << 8;
u32 Zj = 256 << 8;
u32 height = 32;
u8 Angle = 0;

FIXED SIN[256];
FIXED COS[256];
FIXED reciprocal_div[160];


MEM_FUNC_IN_IWRAM void hblFunc()
{
        FIXED Distancia, x_centre_rot, z_centre_rot, dx_mapa, dz_mapa, x0, z0, TempC, TempS, Escalat;
 
 
        // Capturem la lnia de scanline actual...
        int   i = F_VCNT_CURRENT_SCANLINE;
        
        // Guardem valors de sinus cosinus segons l'angle actual, per no fer costosos accessos en va...
        TempC = COS[Angle];
        TempS = SIN[Angle];
    

        // Distancia del punt a terra respecte altura jugador...
        Distancia = ((height*120))*reciprocal_div[i-79] >> 8;       
        

        // Centre de rotaci del mapa...
        x_centre_rot = (TempC*Distancia >> 8) + (Xj);
        z_centre_rot = (TempS*Distancia >> 8) + (Zj);

        // Escalat mapa...
        Escalat = Distancia >> 7;

        // Gradients de mapa...
        dx_mapa =  -TempS*Escalat >> 8;
        dz_mapa =  -TempC*Escalat >> 8;

        // Offset inicial XZ a mapa al scanline actual...
        x0 = x_centre_rot - (dx_mapa << 7);
        z0 = z_centre_rot - (dz_mapa << 7);
        
        // Inicialitzaci de registres (PA i PC)...
        R_BG3ROTDMX = dx_mapa;
        R_BG3ROTDMY = dz_mapa;
//        R_BG3ROTDX = Escalat;
//        R_BG3ROTDY = 0;

        // Inicialitzaci dels registres de punt de referncia...
        R_BG3ROTX = x0;
        R_BG3ROTY = z0;

}


void vblFunc()
{


	newframe=1;
	frames++;
	
	if(F_CTRLINPUT_UP_PRESSED) {Zj+=SIN[Angle];Xj+=COS[Angle];}
	if(F_CTRLINPUT_DOWN_PRESSED) {Zj-=SIN[Angle];Xj-=COS[Angle];}
	if(F_CTRLINPUT_LEFT_PRESSED)    --Angle;
	if(F_CTRLINPUT_RIGHT_PRESSED)   ++Angle;
	if(F_CTRLINPUT_L_PRESSED && height < 1024)   (height++);
	if(F_CTRLINPUT_R_PRESSED && height > 0)   (height--);
	
//	 R_BG3ROTX = Xj;//x0;
//     R_BG3ROTY = Zj;//z0;
     
//     R_BG3ROTDX = COS[Angle];//dx_mapa;
//     R_BG3ROTDY = SIN[Angle];//dz_mapa;
     
    // R_BG3ROTDMX = 512;//dx_mapa;
    // R_BG3ROTDMY = 512;//dz_mapa;

        //R_BG3ROTDMX = 256;
        //R_BG3ROTDMY = 256;


	

	

}



int main(void)
{

	map_fragment_info_ptr mymap3;
	int loop;
	
	for(loop = 0; loop < 160; loop++)
	{
        reciprocal_div[loop] = (FIXED)((1.0f/loop)*65536.0f);
    }

	for(loop = 0; loop < 256; loop++)
	{
        SIN[loop] = (FIXED)(sin(RADIANT(loop*360.0f/256.0f))*256.0f);
        COS[loop] = (FIXED)(cos(RADIANT(loop*360.0f/256.0f))*256.0f);
    }


    // initialize HAMlib
    ham_Init();


    // Inicialitzacio mode rajola amb dos fons rotables i/o escalables
    ham_SetBgMode(2);
    
    // Configura el fons 2 pq sigui inifinit...

    M_BG3CNT_WRAP_SET_OFF;

    // inicialitzaci de les paletes per objectes i el fons
	ham_LoadBGPal(&master_Palette,sizeof(master_Palette));

	// crrega de les rajoles

	ham_bg[3].ti = ham_InitTileSet(&terra_Tiles,SIZEOF_16BIT(terra_Tiles),1,1);
	
	// inicialitzaci d'un mapa buit

	ham_bg[3].mi = ham_InitMapEmptySet(3,1);

	// Es fa una referncia a les dades de mapa en la ROM

     mymap3 = ham_InitMapFragment(&terra_Map,128,128,0,0,128,128,1);
	
    // Cpia (en aquest cas sencera) de mapa al BG2 i a les coordenades x=0 y=0

	ham_InsertMapFragment(mymap3,3,0,0);
	
	// l'habilitem pq es veigi a pantalla

	ham_InitBg(3,1,1,0);
	

//	ham_CreateWin(0,0,80,239,159,WIN_BG3,0,0);


    // inicialitzaci de la rutina d'interrupci VBLANK
    ham_StartIntHandler(INT_TYPE_VBL,&vblFunc);
    ham_StartIntHandler(INT_TYPE_HBL,&hblFunc);
    


    while(1)
    {

    }    
}




