#include "Render.h"

#include "../Dades/LUTs/LUTs.raw.c"

/*******************\
| VARIABLES GLOBALS |
\*******************/

int GRAU_0;  
int GRAU_30; 
int GRAU_45; 
int GRAU_70; 
int GRAU_90; 
int GRAU_110; 
int GRAU_135;
int GRAU_180;
int GRAU_225;
int GRAU_250;
int GRAU_270;
int GRAU_290;
int GRAU_310;
int GRAU_360;

Fixed32      V_SCALE;

int BAIX_DETALL = 1;

/*
Fixed32 CorreccioEfectePeix[AMPLADA_PANTALLA];

Fixed32 Sinus[TAMANY_LUTS];
Fixed32 Cosinus[TAMANY_LUTS];
Fixed32 Tangent[TAMANY_LUTS];
*/

//-----------------------------------------------------------------------------------------//
// ------------------------ INICIALITZACIO RENDER ----------------------------------//

void Render_Init()
{

	// Graus...
	
	GRAU_0  =  0;
	GRAU_30 =  (int)(30.0f/INCREMENT_FOV);
	GRAU_45 =  (int)(45.0f/INCREMENT_FOV);
	GRAU_70 =  (int)(70.0f/INCREMENT_FOV);
	GRAU_90 =  (int)(90.0f/INCREMENT_FOV);
	GRAU_110 =  (int)(110.0f/INCREMENT_FOV);
	GRAU_135 = (int)(135.0f/INCREMENT_FOV);
	GRAU_180 = (int)(180.0f/INCREMENT_FOV);
	GRAU_225 = (int)(225.0f/INCREMENT_FOV);
	GRAU_250 = (int)(250.0f/INCREMENT_FOV);
	GRAU_270 = (int)(270.0f/INCREMENT_FOV);
	GRAU_290 = (int)(290.0f/INCREMENT_FOV);
	GRAU_310 = (int)(310.0f/INCREMENT_FOV);
	GRAU_360 = (int)(360.0f/INCREMENT_FOV);

	V_SCALE = DOUBLE_2_FIXED((float)(  (int)(ALTURA_PANTALLA >> 1) / ( tan( RADIANTS( (FOV)/2.0f ) )) ));
	
}

//-----------------------------------------------------------------------------------------//
// ------------------------ CALCUL DISTANCIA OPTIMITZADA ----------------------------------//


Fixed32 Distancia_2D(Fixed32 x1,Fixed32 z1,Fixed32 x2,Fixed32 z2,int Angle)
{
    Fixed32 x_diff = x1-x2;
    Fixed32 z_diff = z1-z2;
    
    if(abs(x_diff) > abs(z_diff)) // Dividim entre cosinus per evitar una divisi entre 0.
    {
        return abs(DivFx(x_diff,Cosinus[Angle]));
    }
    else //el mateix per amb el sinus...
    {
        return abs(DivFx(z_diff,Sinus[Angle]));
    }
}

//-----------------------------------------------------------------------------------------
//------------------------  IMPLEMENTACI DEL RAIG X --------------------------------------
//-----------------------------------------------------------------------------------------

//{Calcul del punt de partida del raig(Xi,Zi) i dels increments X i Z (Dx,Dz), comentats als apartats 4.1.1 i 4.1.2}
void CalcularPassosRaigX(Fixed32 Xj,Fixed32 Zj, int AngleRaig,Fixed32 *Xi,Fixed32 *Zi,Fixed32 *IncrementX,Fixed32 *IncrementZ)
{
	 Fixed32 ComencamentX;


	 ComencamentX = INT_2_FIXED((FIXED_2_INT(Xj) >> LOG2_AMPLADA_GRAELLA) << LOG2_AMPLADA_GRAELLA);
	 (*IncrementX) = AMPLADA_GRAELLA;
 
	 if((AngleRaig < GRAU_90) || (AngleRaig > GRAU_270)) // {El raig s'orienta a l'est }
	 {

 		 (*Xi) = ComencamentX + AMPLADA_GRAELLA; //{Offset a la graella adjacent dreta d'on es troba el jugador}

		 if(AngleRaig == 0)// {IncrementZ = 0 i Zi = Zj}
		 {

			(*Zi) = Zj;
			(*IncrementZ) = 0;
		 }
		 else //{ s'aplica clculs fets a l'apartat 5.2.1.1 i 5.2.1.2  per esbrinar Zi,Dz}
		 {
  
			(*Zi) = MulFx((*Xi) - Xj,Tangent[AngleRaig]) + Zj;
			(*IncrementZ) = MulFx(*IncrementX,Tangent[AngleRaig]);
		 }
	 }
	 else //{El raig s'orienta a l'est }
	 {
 
		(*Xi) = ComencamentX;// {Offset a la graella on es troba el jugador.}
		(*IncrementX) = - (*IncrementX);

		if(AngleRaig == GRAU_180) //{IncrementZ = 0 i Zi = Zj}
		{

			(*Zi) = Zj;
			(*IncrementZ) = 0;
		}
		else //{ s'aplica clculs fets a l'apartat 5.2.1.1 i 5.2.1.2  esbrinar Zi,Dz}
		{
			
			(*Zi) = MulFx((*Xi)-Xj,Tangent[AngleRaig]) + Zj;
			(*IncrementZ) = MulFx((*IncrementX),Tangent[AngleRaig]);
		}
	}
}


Fixed32 RaigX(int Xj,int Zj,int AngleRaig, int *Textura,int *ColumnaTextura)
{
	int     PosicioMapaX,PosicioMapaZ,Raig_Dins_Mapa,Paret;
	Fixed32   PosicioRaigZ,Zi,IncrementZ,PosicioRaigX, Xi,IncrementX;

	Paret = 0;
	Raig_Dins_Mapa = CERT;

	if ((AngleRaig != GRAU_90) && (AngleRaig != GRAU_270)) //{ Angle vlid }
	{
	
		 CalcularPassosRaigX(Xj,Zj,AngleRaig,&Xi,&Zi,&IncrementX,&IncrementZ);

		 //{S'obt la posici actual del raig en la matriu 2D}
		 PosicioMapaRaigX(Xi,Zi,AngleRaig,&PosicioMapaX,&PosicioMapaZ);

		 //{Inicialitzem les posicions de partida del raig}

		 PosicioRaigX = Xi;
		 PosicioRaigZ = Zi;

		 if(!PuntDinsMapa(PosicioRaigX,PosicioRaigZ)) Raig_Dins_Mapa = FALS;
		 else Paret = Mapa[PosicioMapaZ][PosicioMapaX];

		 if(Paret == 0 && Raig_Dins_Mapa)// { Traem el raig}
		 {
   
			 do{

				PosicioRaigX = PosicioRaigX + IncrementX;
				PosicioRaigZ = PosicioRaigZ + IncrementZ;
				PosicioMapaRaigX(PosicioRaigX,PosicioRaigZ,AngleRaig,&PosicioMapaX,&PosicioMapaZ);

				 if(!PuntDinsMapa(PosicioRaigX,PosicioRaigZ)) Raig_Dins_Mapa = FALS;
				 else Paret = Mapa[PosicioMapaZ][PosicioMapaX];


			 }while(Paret == 0 && Raig_Dins_Mapa );    // Ull! nota que a la documentaci tenim un "fins que..." en ves d'un "mentre...".
						  // Aix noms canvia el negat dels valors a avaluar.
		 }

		 if(Raig_Dins_Mapa)
		 {
 
   
		 		
			//{ Textura }
			(*Textura) = Mapa[PosicioMapaZ][PosicioMapaX]-1;

			//{ Columna de textura }
			(*ColumnaTextura) = FIXED_2_INT(PosicioRaigZ) & ( AMPLADA_TEXTURA - 1);

			//{ Retornem la distncia }   
			return Distancia_2D(Xj,Zj,PosicioRaigX,PosicioRaigZ,AngleRaig);//(sqrt(pow(PosicioRaigX-Xj,2)+pow(PosicioRaigZ-Zj,2)));
		 }
		 else
		 {
			return DISTANCIA_MES_LLARGA;
		 
		 }

	}
	else //{ Error, per que els calculs de tan(90) o tan(270) sn infinits}
	{
			//{ Retornem la distncia ms llarga }

			return DISTANCIA_MES_LLARGA;

	}
}




//------------------------------------------------------------------------------------------------
//------------------------------------- Raig Z ---------------------------------------------------
//------------------------------------------------------------------------------------------------


//{Calcul del punt de partida del raig(Xi,Zi) i dels increments X i Z (Dx,Dz), comentats als apartats 4.1.1 i 4.1.2}
void CalcularPassosRaigZ(Fixed32 Xj,Fixed32 Zj,int AngleRaig,Fixed32 *Xi,Fixed32 *Zi,Fixed32 *IncrementX,Fixed32 *IncrementZ)
{
	Fixed32 ComencamentZ;


	ComencamentZ = INT_2_FIXED((FIXED_2_INT(Zj) >> LOG2_ALTURA_GRAELLA) << LOG2_ALTURA_GRAELLA);
	(*IncrementZ)   = ALTURA_GRAELLA;

	if((AngleRaig < GRAU_180) && (AngleRaig > 0)) // {El raig s'orienta al sud }
	{

      (*Zi) = ComencamentZ + ALTURA_GRAELLA;

	  if (AngleRaig == GRAU_90) //{ IncrementX = 0 i Xi s la posici Xj.}
	  {
		   
		(*Xi) = Xj;
		(*IncrementX) = 0;
	  }
      else// { s'aplica clculs fets a l'apartat 5.2.2.1 i 5.2.2.2  esbrinar Xi,Dx}
	  {
  
		(*Xi) = DivFx((*Zi) - Zj, Tangent[AngleRaig]) + Xj;
		(*IncrementX) = DivFx((*IncrementZ), Tangent[AngleRaig]);
	  }
	}
    else //{ el raig s'orienta al nord  es parteix de la graella on es troba el jugador}
	{
 
		(*Zi) = ComencamentZ;
		(*IncrementZ) = - (*IncrementZ);

		if( AngleRaig == GRAU_270)// { IncrementX = 0 i Xi s la posici Xj}
		{
  
   			(*Xi) = Xj;
			(*IncrementX) = 0;
		}
		else //{ s'aplica clculs fets a l'apartat 5.2.2.1 i 5.2.2.2 per esbrinar Xi,Dx}
		{
  
			(*Xi) = DivFx(((*Zi) - (Zj)), Tangent[AngleRaig]) + Xj;
			(*IncrementX) = DivFx((*IncrementZ), Tangent[AngleRaig]);
		}
	}
}
    


Fixed32 RaigZ(Fixed32 Xj,Fixed32 Zj, int AngleRaig,int *Textura,int *ColumnaTextura)
{

	int   PosicioMapaX,PosicioMapaZ,Raig_Dins_Mapa,Paret;
	Fixed32 PosicioRaigZ,Zi,IncrementZ,PosicioRaigX,Xi,IncrementX;


	Raig_Dins_Mapa = CERT;
	Paret = 0;


	if ((AngleRaig != GRAU_180) && (AngleRaig != 0)) // { Angle vlid }
	{
 
		//{Calculem el punt de partida del raig + els increments X i Z, explicats als apartats 5.2.2.1 i 5.2.2.2}
		CalcularPassosRaigZ(Xj,Zj,AngleRaig,&Xi,&Zi,&IncrementX,&IncrementZ);

		//{Obtenim la posici actual del raig en la matriu 2D}
		PosicioMapaRaigZ(Xi,Zi,AngleRaig,&PosicioMapaX,&PosicioMapaZ);

		//{El punt de partida del raig }

		PosicioRaigX = Xi;
		PosicioRaigZ = Zi;

		if(!PuntDinsMapa(PosicioRaigX,PosicioRaigZ)) Raig_Dins_Mapa = FALS;
		else Paret = Mapa[PosicioMapaZ][PosicioMapaX];

		if (Paret == 0 && Raig_Dins_Mapa)// { Es Traa el raig}
		{
			
			do{

				PosicioRaigX = PosicioRaigX + IncrementX;
				PosicioRaigZ = PosicioRaigZ + IncrementZ;
				PosicioMapaRaigZ(PosicioRaigX,PosicioRaigZ,AngleRaig,&PosicioMapaX,&PosicioMapaZ);

				if(!PuntDinsMapa(PosicioRaigX,PosicioRaigZ)) Raig_Dins_Mapa = FALS;
				 else Paret = Mapa[PosicioMapaZ][PosicioMapaX];
			
			}while  (Paret == 0 && Raig_Dins_Mapa );   // Ull! nota que a la documentaci tenim un "fins que..." en ves d'un "mentre...".
					    // Aix noms canvia el negat dels valors a evaluar.

		}
		
		if(Raig_Dins_Mapa)
		{
		
	
			//{ Textura }

			(*Textura) = Mapa[PosicioMapaZ][PosicioMapaX]-1;

			//{ Columna de textura }

			(*ColumnaTextura) = FIXED_2_INT(PosicioRaigX) & (AMPLADA_TEXTURA-1);

			//{ La distncia }   

			return Distancia_2D(Xj,Zj,PosicioRaigX,PosicioRaigZ,AngleRaig);//(Fixed32)(sqrt(pow(PosicioRaigX-Xj,2)+pow(PosicioRaigZ-Zj,2)));
		}
		else
		{
				return DISTANCIA_MES_LLARGA;
		}
		
	}
	else// { Error, per que els calculs de x/tan(0) o x/tan(180) sn infinits }
	{
		//{ Es retorna la distncia ms llarga }

		return DISTANCIA_MES_LLARGA;
	}
}
//--------------------------------------------------------------------
//-------------------------- DIBUIXAT DE TIRA ------------------------
//--------------------------------------------------------------------




void PintarTira(char *ptr_textura,Fixed32 z3d,int u,int columna)
{

	int y2d,y1;
  	Fixed32 Escalat_v,v1;
  	tParametresScanLineVertical ParametresScanLineVertical;

	char *ptr_video;
	short Color;

	 //{S'obt l'altura de tira de paret projectada, arrodonint cap l'enter ms positiu.}

	y2d = ROUND_FIXED_2_INT(DivFx((V_SCALE << 6), z3d ));

	//{Es calcula l'escalat...}

	Escalat_v = DivFx(INT_2_FIXED(ALTURA_PARET_MON), INT_2_FIXED(y2d));


	//{Es calculen parametres per el scanline vertical...}

	if (y2d > 160) // {Corregim, ja que no podem pintar ms de 160 files}
	{
		y2d = 160;
	}

	
	//{Es calcula l'inici fila de video..}

	y1 = CENTRE_Y2D - (y2d >> 1);

	//{Es calcula l'offset inici fila de textura..}


	v1 = CENTRE_Y3D - MulFx(INT_2_FIXED(y2d >> 1),Escalat_v);

	//{Obtenci del punter de video + offset...}

#ifdef __HAM__


	ptr_video = gba_work_vram + y1*AMPLADA_PANTALLA + columna;

	ParametresScanLineVertical.ptr_video = ptr_video;
	ParametresScanLineVertical.ptr_textura = ptr_textura + u;
	
	ParametresScanLineVertical.v1 = v1;
	ParametresScanLineVertical.Escalat_v = Escalat_v;

	ParametresScanLineVertical.NumeroPixels = y2d;
	
if(BAIX_DETALL)
	ARM_ScanLineVertical_LOW(&ParametresScanLineVertical);
else
    ARM_ScanLineVertical_HIGH(&ParametresScanLineVertical);
  


#endif
#ifdef __SDL__
	ptr_video = sdl_work_vram + y1*AMPLADA_PANTALLA + columna;
	ptr_textura = ptr_textura + u;

	//{bucle general del renderitzat de la tira de paret}
	while( y2d > 0)
	{
		
		Color = ptr_textura[((v1 >> 8) << 6)];
		
		if(!BAIX_DETALL)
		{
			*(ptr_video) = (char)Color;		
		}
		else
		{
			Color |= (Color << 8);
			*((short *)ptr_video) = Color;
		}
		
		

		ptr_video += 240;
		v1 = v1 + Escalat_v;
		y2d --;
	}


#endif








}



//------------------------------------------------------------------------
//------------------------ ALGORISME DE RENDER ---------------------------
//------------------------------------------------------------------------


void Renderitzar(Fixed32 PosicioJugadorX,Fixed32 PosicioJugadorZ,int DireccioJugador)
{
	Fixed32 DistanciaRaigX, DistanciaRaigZ,Distancia;
	int AngleRaig,ColumnaActual, TexturaX, TexturaZ, uX, uZ;
	 

  if(!JugadorDinsMapa((PosicioJugadorX),(PosicioJugadorZ)))
  {
	  return;
  }

  //{1.- Basat en el FOV que tenim,  es resta 30 (meitat del FOV)}

  AngleRaig = DireccioJugador - GRAU_30;

  if (AngleRaig < 0)  //{ Estem per sota els 360, cal corregir}
		AngleRaig = AngleRaig + GRAU_360;
   

  for(ColumnaActual = 0; ColumnaActual < AMPLADA_PANTALLA; ColumnaActual+= (1 << BAIX_DETALL))
  {

	//{ Llencem els dos raigs }

	
	DistanciaRaigX = RaigX(PosicioJugadorX,PosicioJugadorZ,AngleRaig,&TexturaX,&uX);
	DistanciaRaigZ = RaigZ(PosicioJugadorX,PosicioJugadorZ,AngleRaig,&TexturaZ,&uZ);

	//{ Pintarem la tira de textura de la distncia de paret ms propera al jugador }

	if (DistanciaRaigX < DistanciaRaigZ) // {Pintem la tira del raig X}
	{

		//{Corregim distncia}

		Distancia = MulFx(DistanciaRaigX,CorreccioEfectePeix[ColumnaActual]);

	    PintarTira(&Textura[TexturaX][0][0],Distancia,uX,ColumnaActual);
	}
	else //{Pintem la tira del raig Z }
	{
	
		//{Corregim distncia}
		Distancia = MulFx(DistanciaRaigZ,CorreccioEfectePeix[ColumnaActual]);

		PintarTira(&Textura[TexturaZ][0][0],Distancia,uZ,ColumnaActual);

	}

	AngleRaig = AngleRaig + (1 << BAIX_DETALL); // {IncrementAngle = 0.250}

	if (AngleRaig >= GRAU_360) // { Si estem igual o per sobre els 360 ,cal corregir  }
     AngleRaig = AngleRaig - GRAU_360;
  }

}


