.ARM            @select 32bit ARM instruction set

@---
.data

msg_boot:
 .ascii  "BOOT"     @\
msg_okay:
 .ascii  "OKAY"     @ ID codes for the burstboot protocol
msg_brst:
 .ascii  "BRST"     @/
@------------------


@.gba            @indicate that it's a gameboy advance program
@.fix            @automatically fix the cartridge header checksum

@org #33554432    @origin in RAM for multiboot-cable/no$gba-cutdown compatibility

@------------------
@cartridge header/multiboot header
 b     rom_start                @-rom entry point
 .byte   0,0,0,0,0,0,0,0,0,0   @-nintento logo (156 bytes)
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0,0,0,0,0
 .byte   0,0,0,0,0,0

 .byte   'XBOO SAMPLE '           @-title (12 bytes)
 .byte   0,0,0,0,0,0              @-game code (4 bytes), maker code (2 bytes)
 .byte   150,0,0                  @-fixed value 96h, main unit code, device type
 .byte   0,0,0,0,0,0,0            @-reserved (7 bytes)
 .byte   0                        @-software version number
 .byte   0                        @-header checksum (set by .fix)
 .byte   0,0                      @-reserved (2 bytes)
  b     ram_start                 @-multiboot ram entry point
 .byte   0,0                      @-multiboot reserved bytes (destroyed by BIOS)
 .byte   0,0                      @-blank padded (32bit alignment)
@------------------
irq_handler:  @interrupt handler (note: r0-r3 are pushed by BIOS)
 mov    r1,#67108864             @\get I/O base address,
 ldr    r0,[r1,#512] @IE/IF     @ read IE and IF,
 and    r0,r0,r0,lsr #16         @ isolate occurred AND enabled irqs,
 add    r3,r1,#512   @IF        @ and acknowledge these in IF
 strh   r0,[r3,#2]               @/
 ldrh   r3,[r1,#-8]              @\mix up with BIOS irq flags at 3007FF8h,
 orr    r3,r3,r0                @ aka mirrored at 3FFFFF8h, this is required
 strh   r3,[r1,#-8]              @/when using the (VBlank-)IntrWait functions
 and    r3,r0,#128 @IE/IF.7 SIO  @\
 cmp    r3,#128                  @ check if it's a burst boot interrupt
 ldreq  r2,[r1,#288] @SIODATA32 @ (if interrupt caused by serial transfer,
 ldreq  r3,msg_brst           @ and if received data is "BRST",
 cmpeq  r2,r3                   @ then jump to burst boot)
 b      burst_boot              @/
                                @... insert your own interrupt handler code here ...
 bx     lr                      @-return to the BIOS interrupt handler
@------------------
burst_boot:                     @requires incoming r1=4000000h

@... if your program uses DMA, disable any active DMA transfers here ...

 ldr   r4,msg_okay            @\
 bl    sio_transfer             @ receive transfer length/bytes & reply "OKAY"
 mov   r2,r0 @len               @/
 mov   r3,#50331648   @dst       @\
 mov   r4,#0  @crc               @
__lop:                          @

 bl    sio_transfer             @ download burst loader to #50331648 and up
 stmia r3!,{r0}      @dst       @
 add   r4,r4,r0      @crc       @
 subs  r2,r2,#4       @len       @
 bhi   __lop                    @/
 bl    sio_transfer             @-send crc value to master
 @b     $3000000               @ARM state!    @-launch actual transfer by starting the loader

@------------------

sio_transfer:  @serial transfer subroutine, 32bit normal mode, external clock
 str   r4,[r1,#288]  @siodata32 @-set reply/send data
 ldr   r0,[r1,#296]  @siocnt    @\
 orr   r0,r0,#128                @ activate slave transfer
 str   r0,[r1,#296]  @siocnt    @/
__wait:                         @\
 ldr   r0,[r1,#296]  @siocnt    @ wait until transfer completed
 tst   r0,#128                   @
 bne   __wait                   @/
 ldr   r0,[r1,#288]  @siodata32 @-get received data
 bx    lr

@--------------------

download_rom_to_ram:

 mov  r0,#2147483648             @src/rom     @\
 mov  r1,#33554432               @dst/ram     @
 mov  r2,#0x40000/16                  @length      @ transfer the ROM content

__lop2:                          @ into RAM (done in units of 4 words/16 bytes)

 ldmia r0!,{r4,r5,r6,r7}        @ currently fills whole 256K of RAM,
 stmia r1!,{r4,r5,r6,r7}        @ even though the proggy is smaller
 subs  r2,r2,#1                  @
 bne   __lop2                    @/

 sub   r15,lr,#0x8000000-0x2000000 @#2147483648-#33554432 @-return (retadr rom/8000XXXh -> ram/2000XXXh)

@------------------

init_interrupts:

 mov  r4,#67108864               @-base address for below I/O registers
 ldr  r0,=irq_handler           @\install IRQ handler address
 str  r0,[r4,#-4]   @IRQ HANDLER @/at 3FFFFFC aka 3007FFC
 mov  r0,#8                  @\enable generating vblank irqs
 strh r0,[r4,#4]   @DISPSTAT    @/
 mrs  r0,cpsr                   @\
 bic  r0,r0,#128                @ cpu interrupt enable (clear i-flag)
 msr  cpsr,r0                   @/
 mov  r0,#0                     @\
 str  r0,[r4,#308] @RCNT        @ init SIO normal mode, external clock,
 ldr  r0,=0x5080                @ 32bit, IRQ enable, transfer started
 str  r0,[r4,#296] @SIOCNT      @ output "BOOT" (indicate burst boot prepared)
 ldr  r0,msg_boot             @
 str  r0,[r4,#288] @SIODATA32   @/
 mov  r0,#1                      @\interrupt master enable
 str  r0,[r4,#520] @IME=1       @/
 mov  r0,#129                    @\enable execution of vblank IRQs,
 str  r0,[r4,#512] @IE=#129      @/and of SIO IRQs (burst boot)
 bx   lr
@------------------
rom_start:   @entry point when booted from flashcart/rom
 bl   download_rom_to_ram       @-download ROM to RAM (returns to ram_start)
ram_start:   @entry point for multiboot/burstboot
 mov  r0,#254                   @\reset all registers, and clear all memory
 swi  #65536 @RegisterRamReset  @/(except program code in wram at #33554432)
 bl   init_interrupts           @-install burst boot irq handler
 mov  r4,#67108864               @\enable video,
 strh r4,[r4,#0] @DISPCNT     @/by clearing the forced blank bit
__mainloop:
 swi  #327680 @VBlankIntrWait    @-wait one frame (cpu in low power mode)
 mov  r5,#83886080               @\increment the backdrop palette color
 str  r8,[r5]                   @ (ie. display a blinking screen)
 add  r8,r8,#1                   @/
 b    __mainloop
@------------------
.pool

.end
